/*
    This file is part of the FElt finite element analysis package.
    Copyright (C) 1993,1994 Jason I. Gobat and Darren C. Atkinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/************************************************************************
 * File:	analysis.c						*
 *									*
 * Description:	This file contains the private and public function and	*
 *		type definitions for the analysis parameters dialog	*
 *		box.							*
 ************************************************************************/

# include <stdio.h>
# include <X11/Intrinsic.h>
# include <X11/StringDefs.h>
# include <X11/Shell.h>
# include <X11/Xaw/AsciiText.h>
# include <X11/Xaw/Command.h>
# include <X11/Xaw/Toggle.h>
# include <X11/Xaw/Label.h>
# include "Layout.h"
# include "TabGroup.h"
# include "Analysis.h"
# include "util.h"
# include "problem.h"
# include "allocate.h"
# include "procedures.h"

# ifndef X_NOT_STDC_ENV
# include <stdlib.h>
# else
extern int    atoi ( );
# endif

static unsigned		base;

struct analysis_dialog {
    Widget  shell;		/* topLevelShell  <specified>	*/
    Widget  layout;		/*	Layout  layout		*/
    Widget  dt;			/*           AsciiText dt	*/
    Widget  duration;		/*           AsciiText duration */
    Widget  beta;		/*	     AsciiText beta	*/
    Widget  gamma;		/*           AsciiText gamma	*/
    Widget  alpha;		/*           AsciiText alpha    */
    Widget  dof [6];		/*	     Toggle dof [1-6]	*/
    Widget  consistent;		/*           Toggle consistent  */
    Widget  lumped;		/*           Toggle lumped      */ 
    Widget  left;		/*     	     Command left	*/
    Widget  right;		/*	     Command right	*/
    Widget  node [6];		/*	     AsciiText node[1-6]*/
    Widget  help;               /*           MenuButton  help   */
    Widget  accept;             /*           Command  accept    */
    Widget  animate;		/*           Command  animate   */
    Widget  dismiss;            /*           Command  dismiss   */
};


static String labels [ ] = {
    "dt:","t:","b:","g:","a:","DOFs","Tx","Ty","Tz","Rx","Ry","Rz",
    "consistent mass matrix","lumped mass matrix","Nodes:"
};

static String label_names [ ] = {
    "dt_name","duration_name","beta_name","gamma_name","alpha_name",
    "dofs_name","tx_name","ty_name","tz_name","rx_name","ry_name",
    "rz_name","consistent_name","lumped_name","nodes_name"
};


/* Resources */

static Pixel highlight;

static char layout_string [ ] =
"vertical { \
    horizontal { \
       vertical { \
          4 \
          ((height duration - height duration_name) / 2) \
          duration_name \
          ((height duration - height duration_name) / 2) \
          4 \
          ((height dt - height dt_name) / 2) \
          dt_name \
          ((height dt - height dt_name) / 2) \
          4 \
          ((height beta - height beta_name) / 2) \
          beta_name \
          ((height beta - height beta_name) / 2) \
          4 \
          ((height gamma - height gamma_name) / 2) \
          gamma_name \
          ((height gamma - height gamma_name) / 2) \
          4 \
          ((height alpha - height alpha_name) / 2) \
          alpha_name \
          ((height alpha - height alpha_name) / 2) \
          4 \
       } \
       1 \
       vertical { \
          4 \
          duration \
          4 \
          dt \
          4 \
          beta \
          4 \
          gamma \
          4 \
          alpha \
          4 \
       } \
       8 \
       separator1 <* +inf -100%> \
       8 \
       vertical { \
          ((height duration - height dofs_name) / 2) \
          horizontal { \
             ((width dof1 + width tx_name + width dof4 + width rx_name + \
               64 - width dofs_name) / 2) \
             dofs_name \
             ((width dof1 + width tx_name + width dof4 + width rx_name + \
               64 - width dofs_name) / 2) \
          } \
          ((height duration - height dofs_name) / 2) \
          8 \
          horizontal { \
             tx_name \
             4 \
             dof1 \
             52 \
             rx_name \
             4 \
             dof4 \
             4 \
          } \
          8 \
          horizontal { \
             ty_name \
             4 \
             dof2 \
             52 \
             ry_name \
             4 \
             dof5 \
             4 \
          } \
          8 \
          horizontal { \
             tz_name \
             4 \
             dof3 \
             52 \
             rz_name \
             4 \
             dof6 \
             4 \
          } \
          4 \
       } \
       4 \
    } \
    separator2 <+inf -100% *> \
    4 \
    horizontal { \
       4 \
       consistent \
       4 \
       consistent_name \
       16 \
       lumped \
       4 \
       lumped_name \
       4 \
    } \
    4 \
    separator3 <+inf -100% *> \
    4 \
    horizontal { \
       4 \
       left \
       4 \
       right \
       8 \
       nodes_name \
       4 \
       node1 \
       4 \
       node2 \
       4 \
       node3 \
       4 \
       node4 \
       4 \
       node5 \
       4 \
       node6 \
       4 \
    } \
    4 \
    separator4 <+inf -100% *> \
    4 \
    horizontal { \
       4 \
       help \
       4 <+inf -100%> \
       animate \
       4 <+inf -100%> \
       accept \
       4 <+inf -100%> \
       dismiss \
       4 \
    } \
    4 \
}";

static Arg color_args [ ] = {
    {XtNborderColor, (XtArgVal) &highlight},
};

static Arg shell_args [ ] = {
    {XtNtitle,    (XtArgVal) NULL},
    {XtNiconName, (XtArgVal) NULL},
};

static Arg layout_args [ ] = {
    {XtNlayout, (XtArgVal) NULL},
};

static Arg text_args [ ] = {
    {XtNeditType,    (XtArgVal) XawtextEdit},
    {XtNborderWidth, (XtArgVal) 0},
    {XtNpieceSize,   (XtArgVal) 32},
    {XtNcursorName,  (XtArgVal) "left_ptr"},
};

static Arg node_args [ ] = {
    {XtNeditType,    (XtArgVal) XawtextEdit},
    {XtNborderWidth, (XtArgVal) 0},
    {XtNpieceSize,   (XtArgVal) 32},
    {XtNcursorName,  (XtArgVal) "left_ptr"},
    {XtNwidth,       (XtArgVal) 40},
};

static Arg bitmap_args [ ] = {
    {XtNbitmap, (XtArgVal) NULL},
};

static Arg label_args [ ] = {
    {XtNlabel,	            (XtArgVal) ""},
    {XtNborderWidth,        (XtArgVal) 0},
};

static Arg core_args [ ] = {
    {XtNwidth,	(XtArgVal) 1},
    {XtNheight, (XtArgVal) 1},
};

static Arg toggle_args [ ] = {
    {XtNlabel,		    (XtArgVal) " "},
};

#define left_width 12
#define left_height 12
static char left_bits[] = {
   0x00, 0x00, 0x80, 0x01, 0xc0, 0x01, 0xe0, 0x01, 0xf0, 0x01, 0xf8, 0x01,
   0xf8, 0x01, 0xf0, 0x01, 0xe0, 0x01, 0xc0, 0x01, 0x80, 0x01, 0x00, 0x00};

#define right_width 12
#define right_height 12
static char right_bits[] = {
   0x00, 0x00, 0x18, 0x00, 0x38, 0x00, 0x78, 0x00, 0xf8, 0x00, 0xf8, 0x01,
   0xf8, 0x01, 0xf8, 0x00, 0x78, 0x00, 0x38, 0x00, 0x18, 0x00, 0x00, 0x00};

/* Translation tables */

static String text_table =
"<Key>Return: AnalysisDialogAction(accept)\n\
 <Key>Escape: AnalysisDialogAction(dismiss)\n\
 Ctrl<Key>h:  AnalysisDialogAction(help)\n\
 <Btn1Down>:  SetFocus() select-start()";

static XtTranslations text_translations;

static String command_table =
"<Key>Return:   AutoRepeat(off) set()\n\
 <KeyUp>Return: AutoRepeat(saved) notify() unset()\n\
 <Key>space:    AutoRepeat(off) set()\n\
 <KeyUp>space:  AutoRepeat(saved) notify() unset()\n\
 <Key>Escape:   AnalysisDialogAction(dismiss)";

static XtTranslations command_translations;

static String toggle_table =
"<Key>Return:	AnalysisDialogAction(accept)\n\
 <Key>Escape:	AnalysisDialogAction(dismiss)\n\
 <Key>space:	toggle()\n\
 Ctrl<Key>h:	AnalysisDialogAction(help)\n\
 <Btn1Down>:	SetFocus()";

static XtTranslations toggle_translations;

static String help_table =
"<Key>Return: AnalysisDialogAction(accept)\n\
 <Key>Escape: AnalysisDialogAction(dismiss)\n\
 Ctrl<Key>d:  AnalysisDialogAction(delete)\n\
 Ctrl<Key>c:  AnalysisDialogAction(copy)\n\
 Ctrl<Key>n:  AnalysisDialogAction(new)\n\
 Ctrl<Key>h:  AnalysisDialogAction(help)\n\
 <Key>space:  PostMenu()";

static XtTranslations help_translations;

/* Help message. */

static String help_message ="\
Use the analysis parameters dialog to control the solution of transient \
analysis problems.  Beta, gamma, and alpha are numerical parameters \
in the Hilbert-Hughes-Taylor variant of the Newmark method for integration \
over time.  dt and t define the time step and total time length of the \
numerical simulation.  Use the node entries to define which noes you would \
like to see results for.  The DOFs toggle control the degrees of freedom \
which will be analyzed at each of these nodes. \
";


/************************************************************************
 * Function:	Action							*
 *									*
 * Description:	An action procedure which emulates pressing of the	*
 *		specified button.					*
 ************************************************************************/

static void Action (w, event, params, num_params)
    Widget    w;
    XEvent   *event;
    String   *params;
    Cardinal *num_params;
{
    if (XtClass (w) == topLevelShellWidgetClass)
	w = XtNameToWidget (w, "layout.dismiss");
    else
	w = XtNameToWidget (XtParent (w), params [0]);

    XtCallCallbacks (w, XtNcallback, NULL);
}


/************************************************************************
 * Function:	Accept							*
 *									*
 * Description:								*
 ************************************************************************/

static void Accept (w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Arg		   args [1];
    AnalysisDialog analysisd;
    String	   value;
    Boolean	   state;
    unsigned	   i;

    analysisd = (AnalysisDialog) client_data;

    XtSetArg (args [0], XtNstring,  &value);

    XtGetValues (analysisd -> dt, args, 1);
    analysis.dt = exptod (value, NULL);
    XtGetValues (analysisd -> duration, args, 1);
    analysis.duration = exptod (value, NULL);
    XtGetValues (analysisd -> beta, args, 1);
    analysis.beta = exptod (value, NULL);
    XtGetValues (analysisd -> gamma, args, 1);
    analysis.gamma = exptod (value, NULL);
    XtGetValues (analysisd -> alpha, args, 1);
    analysis.alpha = exptod (value, NULL);

    XtSetArg (args [0], XtNstate, &state);

    analysis.numdofs = 0;
    for (i = 0 ; i < 6 ; i++) {
       XtGetValues (analysisd -> dof [i], args, 1);
       if (state) 
          analysis.dofs [++ analysis.numdofs] = i + 1;
    }

    XtGetValues (analysisd -> consistent, args, 1);
    if (state)
       analysis.mass_mode = 'c';
    else {
       XtGetValues (analysisd -> lumped, args, 1);
       if (state)
          analysis.mass_mode = 'l';
       else
          analysis.mass_mode = 0;
    }

    AnalysisDialogUpdate (analysisd, False);
}


/************************************************************************
 * Function:	ShiftNodes						*
 *									*
 * Description:	Shuffle the node entries left and right			*
 ************************************************************************/

static void ShiftNodes (w, client_data, call_data)
    Widget	w;
    XtPointer	client_data;
    XtPointer	call_data;
{
    Arg		    args [1];
    AnalysisDialog  analysisd;
    unsigned	    current [6];
    unsigned	    count;
    String	    value;
    char	    buffer [10];
    unsigned	    node;
    unsigned	    i;

    analysisd = (AnalysisDialog) client_data;

   	/*
	 * figure out which entries have numbers in them
	 */

    XtSetArg (args [0], XtNstring,  &value);

    count = 0;
    for (i = 0 ; i < 6 ; i++) {
       XtGetValues (analysisd -> node[i], args, 1);
       node = atoi (value);
       if (node != 0) 
          current [count++] = node;
    }

	/*
	 * if there are new ones, make some room for them
	 */

    if (base + count > analysis.numnodes) {
       analysis.numnodes = base + count;
       ZeroOffset (analysis.nodes);
       analysis.nodes = Reallocate (analysis.nodes,unsigned,analysis.numnodes);
       UnitOffset (analysis.nodes);
    }
    else if (base + count < analysis.numnodes && base + count > 0)
       analysis.numnodes = base + count;

	/*	
 	 * update the nodes array 
         */

    for (i = 0 ; i < count ; i++) 
       analysis.nodes [base + i + 1] = current [i];
       
	/*
	 * if necessary (and allowed) actually do the shift
	 */

    if (w == analysisd -> right && count > 1) 
       base ++;
    else if (w == analysisd -> left && base > 0) 
       base --;

	/*
	 * update the nodes that are actually displayed
	 */

    for (i = 0 ; i < 6 ; i++) {
       if (base + i < analysis.numnodes)
          sprintf (buffer,"%d", analysis.nodes[base+i+1]);
       else
	  buffer [0] = 0;

       SetTextString (analysisd -> node[i], buffer);
    } 
}

/************************************************************************
 * Function:	CheckToggle						*
 *									*
 * Description:	enforces radio behavior on the mass mode toggles	*
 ************************************************************************/

static void CheckToggle (w, client_data, call_data)
    Widget	w;
    XtPointer	client_data;
    XtPointer	call_data;
{
    AnalysisDialog	analysisd;
    Boolean		state;
    Arg			args [1];

    analysisd = (AnalysisDialog) client_data;

    XtSetArg (args [0], XtNstate, &state);
    XtGetValues (w, args, 1);

	/*
	 * we enforce radio behavior by only doing anything if the user
	 * is trying to turn something on
	 */

    if (state) {
       XtSetArg (args [0], XtNstate, False); 
       if (w == analysisd -> lumped) 
          XtSetValues (analysisd -> consistent, args, 1);
       else 
          XtSetValues (analysisd -> lumped, args, 1);
    }
    else {
       XtSetArg (args [0], XtNstate, True);
       XtSetValues (w, args, 1);
    }
}

/************************************************************************
 * Function:	Dismiss							*
 *									*
 * Description:	sets the dismiss flag					*
 ************************************************************************/

static void Dismiss (w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    AnalysisDialog analysisd;

    analysisd = (AnalysisDialog) client_data;
    XtPopdown (analysisd -> shell);
}

/************************************************************************
 * Function:	ExternalCallback					*
 *									*
 * Description:	calls a function outside the immediate control of 	*
 *		the dialog						*
 ************************************************************************/

static void ExternalCallback (w, client_data, call_data)
    Widget 	w;
    XtPointer	client_data;
    XtPointer	call_data;
{
    AnalysisDialog	analysisd;

    analysisd = (AnalysisDialog) client_data;

    Accept (NULL, client_data, NULL);

    if (w == analysisd -> animate) 
       SetupAnimate ( );
}

/************************************************************************
 * Function:	AnalysisDialogUpdate					*
 *									*
 * Description:	sets the widgets based on current analysis settings 	*
 ************************************************************************/

# if defined(__STDC__)
void AnalysisDialogUpdate (AnalysisDialog analysisd, Boolean force)
# else
void AnalysisDialogUpdate (analysisd,force)
    AnalysisDialog	analysisd;
    Boolean		force;
# endif
{
    Arg		args [1];
    char	buffer [80];
    unsigned	i;

	/*
	 * set the text entries
	 */

    sprintf (buffer,"%g",analysis.dt);
    SetTextString (analysisd -> dt, buffer);
    sprintf (buffer,"%g",analysis.duration);
    SetTextString (analysisd -> duration, buffer);
    sprintf (buffer,"%g",analysis.beta);
    SetTextString (analysisd -> beta, buffer);
    sprintf (buffer,"%g",analysis.gamma);
    SetTextString (analysisd -> gamma, buffer);
    sprintf (buffer,"%g",analysis.alpha);
    SetTextString (analysisd -> alpha, buffer);

	/*
	 * set the dof toggles
	 */

    XtSetArg (args [0], XtNstate, False);
    for (i = 0 ; i < 6 ; i++)
       XtSetValues (analysisd -> dof [i], args, 1);

    XtSetArg (args [0], XtNstate, True);
    for (i = 1 ; i <= analysis.numdofs ; i++) 
       XtSetValues (analysisd -> dof [analysis.dofs[i] - 1], args, 1);

	/*
	 * set the mass mode toggles
	 */

    XtSetArg (args [0], XtNstate, True);
    if (analysis.mass_mode == 'c') {
       XtSetValues (analysisd -> consistent, args, 1);
       XtSetArg (args [0], XtNstate, False);
       XtSetValues (analysisd -> lumped, args, 1);
    }
    else if (analysis.mass_mode == 'l') {
       XtSetValues (analysisd -> lumped, args, 1);
       XtSetArg (args [0], XtNstate, False);
       XtSetValues (analysisd -> consistent, args, 1);
    }
    else {
       XtSetArg (args [0], XtNstate, False);
       XtSetValues (analysisd -> lumped, args, 1);
       XtSetValues (analysisd -> consistent, args, 1);
    }

	/*
	 * fill in the nodes
	 */

    if (force) {
       for (i = 0 ; i < 6 ; i++) 
          SetTextString (analysisd -> node[i], "");
    }

    ShiftNodes (NULL, (XtPointer) analysisd, NULL);
}

/************************************************************************
 * Function:	AnalysisDialogCreate					*
 *									*
 * Description:	Creates a new analysis dialog.  You would never want to	*
 *		have more than one of these but the interface is kept	*
 *		consistent with those of the other dialogs.		*
 ************************************************************************/

AnalysisDialog AnalysisDialogCreate (parent, name, title)
    Widget parent;
    String name;
    String title;
{
    char		buffer [6];
    Cardinal		i;
    Arg			args [1];
    Widget		group [25];
    AnalysisDialog	analysisd;
    Dimension		width;
    Position		x;
    Pixmap		left_bitmap;
    Pixmap		right_bitmap;
    static XtAppContext	app_context = NULL;
    static XtActionsRec	actions [ ] = {{"AnalysisDialogAction", Action}};


    /* Perform one time initialization. */

    if (app_context == NULL) {
	app_context = XtWidgetToApplicationContext (parent);
	XtAppAddActions (app_context, actions, XtNumber (actions));
	AddAutoRepeatAction (app_context);

        layout_args [0].value = StringToLayout (parent, layout_string);

        toggle_translations = XtParseTranslationTable (toggle_table);
	text_translations = XtParseTranslationTable (text_table);
	command_translations = XtParseTranslationTable (command_table);
        help_translations = XtParseTranslationTable (help_table);

        
        left_bitmap = XCreateBitmapFromData (XtDisplay (parent), 
                      RootWindowOfScreen (XtScreen (parent)),
                      left_bits, left_width, left_height);
        right_bitmap = XCreateBitmapFromData (XtDisplay (parent), 
                      RootWindowOfScreen (XtScreen (parent)),
                      right_bits, right_width, right_height);
    }


    /* Create the analysis dialog and its widgets. */

    XtSetArg (shell_args [0], XtNtitle,    title);
    XtSetArg (shell_args [1], XtNiconName, title);

    analysisd = XtNew (struct analysis_dialog);

    analysisd -> shell    = XtCreatePopupShell (name,
			 topLevelShellWidgetClass, parent,
			 shell_args, XtNumber (shell_args));

    analysisd -> layout   = XtCreateManagedWidget ("layout",
			 layoutWidgetClass, analysisd -> shell,
			 layout_args, XtNumber (layout_args));

    analysisd -> dt	  = XtCreateManagedWidget ("dt",
                         asciiTextWidgetClass, analysisd -> layout,
                         text_args, XtNumber (text_args));

    analysisd -> duration = XtCreateManagedWidget ("duration",
                         asciiTextWidgetClass, analysisd -> layout,
                         text_args, XtNumber (text_args));

    analysisd -> beta	  = XtCreateManagedWidget ("beta",
                         asciiTextWidgetClass, analysisd -> layout,
                         text_args, XtNumber (text_args));

    analysisd -> gamma	  = XtCreateManagedWidget ("gamma",
                         asciiTextWidgetClass, analysisd -> layout,
                         text_args, XtNumber (text_args));

    analysisd -> alpha	  = XtCreateManagedWidget ("alpha",
                         asciiTextWidgetClass, analysisd -> layout,
                         text_args, XtNumber (text_args));

    analysisd -> consistent = XtCreateManagedWidget ("consistent",
                         toggleWidgetClass, analysisd -> layout,
                         toggle_args, XtNumber (toggle_args));

    analysisd -> lumped   = XtCreateManagedWidget ("lumped",
                         toggleWidgetClass, analysisd -> layout,
                         toggle_args, XtNumber (toggle_args));

    bitmap_args [0].value = (XtArgVal) left_bitmap; 
    analysisd -> left     = XtCreateManagedWidget ("left",
                          commandWidgetClass, analysisd -> layout,
                          bitmap_args, XtNumber (bitmap_args));
 
    bitmap_args [0].value = (XtArgVal) right_bitmap; 
    analysisd -> right    = XtCreateManagedWidget ("right",
                          commandWidgetClass, analysisd -> layout,
                          bitmap_args, XtNumber (bitmap_args));
 
    analysisd -> accept   = XtCreateManagedWidget ("accept",
			 commandWidgetClass, analysisd -> layout,
			 NULL, 0);

    analysisd -> animate   = XtCreateManagedWidget ("animate",
			 commandWidgetClass, analysisd -> layout,
			 NULL, 0);

    analysisd -> dismiss  = XtCreateManagedWidget ("dismiss",
			 commandWidgetClass, analysisd -> layout,
			 NULL, 0);

    for (i = 0 ; i < 6 ; i++) {
       sprintf (buffer, "dof%d", i+1);
       analysisd -> dof[i] = XtCreateManagedWidget (buffer,
                         toggleWidgetClass, analysisd -> layout,
                         toggle_args, XtNumber (toggle_args));
    }

    for (i = 0 ; i < 6 ; i++) {
       sprintf (buffer,"node%d", i+1);
       analysisd -> node[i] = XtCreateManagedWidget (buffer,
                         asciiTextWidgetClass, analysisd -> layout,
                         node_args, XtNumber (node_args)); 
    }

    analysisd -> help     = CreateHelpButton (analysisd -> layout, "help");

    for (i = 0 ; i < XtNumber (labels) ; i++) {
        label_args [0].value = (XtArgVal) labels [i];
        XtCreateManagedWidget (label_names [i], labelWidgetClass,
                   analysisd -> layout, label_args, XtNumber (label_args));
    }

    XtCreateManagedWidget ("separator1", coreWidgetClass, 
               analysisd -> layout, core_args, XtNumber (core_args));
    XtCreateManagedWidget ("separator2", coreWidgetClass, 
               analysisd -> layout, core_args, XtNumber (core_args));
    XtCreateManagedWidget ("separator3", coreWidgetClass, 
               analysisd -> layout, core_args, XtNumber (core_args));
    XtCreateManagedWidget ("separator4", coreWidgetClass, 
               analysisd -> layout, core_args, XtNumber (core_args));


    /* Create a tab group for the analysis dialog. */

    group [0]  = analysisd -> duration;
    group [1]  = analysisd -> dt;
    group [2]  = analysisd -> beta;
    group [3]  = analysisd -> gamma;
    group [4]  = analysisd -> alpha;
    group [5]  = analysisd -> dof[0];
    group [6]  = analysisd -> dof[1];
    group [7]  = analysisd -> dof[2];
    group [8]  = analysisd -> dof[3] ;
    group [9]  = analysisd -> dof[4];
    group [10] = analysisd -> dof[5];
    group [11] = analysisd -> consistent;
    group [12] = analysisd -> lumped;
    group [13] = analysisd -> left;
    group [14] = analysisd -> right;
    group [15] = analysisd -> node[0];
    group [16] = analysisd -> node[1];
    group [17] = analysisd -> node[2];
    group [18] = analysisd -> node[3];
    group [19] = analysisd -> node[4];
    group [20] = analysisd -> node[5];
    group [21] = analysisd -> help;
    group [22] = analysisd -> animate;
    group [23] = analysisd -> accept;
    group [24] = analysisd -> dismiss;

    XtGetValues (analysisd -> layout, color_args, XtNumber (color_args));
    CreateTabGroup (analysisd -> shell, group, XtNumber (group), 
                    highlight, True);

    XtRealizeWidget (analysisd -> shell);
    SetFocus (analysisd -> duration);

    XtSetArg (args [0], XtNwidth, &width);
    XtGetValues (analysisd -> layout, args, 1);
    XtSetArg (args [0], XtNx, &x);
    XtGetValues (analysisd -> help, args, 1);
    UpdateHelpMessage (analysisd -> help, help_message, width - 2 * x);


    /* Add the translations to each widget. */

    AddDeleteWindowProtocol   (analysisd -> shell, "AnalysisDialogAction()");

    XtOverrideTranslations (analysisd -> dt,	    text_translations);
    XtOverrideTranslations (analysisd -> duration,  text_translations);
    XtOverrideTranslations (analysisd -> alpha,     text_translations);
    XtOverrideTranslations (analysisd -> beta,      text_translations);
    XtOverrideTranslations (analysisd -> gamma,     text_translations);
    XtOverrideTranslations (analysisd -> consistent,toggle_translations);
    XtOverrideTranslations (analysisd -> lumped,    toggle_translations);
    XtOverrideTranslations (analysisd -> left,      command_translations);
    XtOverrideTranslations (analysisd -> right,     command_translations);
    XtOverrideTranslations (analysisd -> dismiss,   command_translations);
    XtOverrideTranslations (analysisd -> accept,    command_translations);
    XtOverrideTranslations (analysisd -> animate,    command_translations);
    XtOverrideTranslations (analysisd -> help,	    help_translations);
    for (i = 0 ; i < 6 ; i++) {
       XtOverrideTranslations (analysisd -> node[i], text_translations);
       XtOverrideTranslations (analysisd -> dof[i],  toggle_translations);
    }
    

    /* Add the necessary callbacks. */

    XtAddCallback(analysisd->accept, XtNcallback,Accept, (XtPointer) analysisd);
    XtAddCallback(analysisd->dismiss,XtNcallback,Dismiss,(XtPointer) analysisd);
    XtAddCallback(analysisd->consistent, XtNcallback,
                  CheckToggle, (XtPointer)analysisd);
    XtAddCallback(analysisd->lumped, XtNcallback,
                  CheckToggle, (XtPointer)analysisd);
    XtAddCallback(analysisd->animate, XtNcallback,
                  ExternalCallback, (XtPointer) analysisd);
    XtAddCallback(analysisd->left, XtNcallback, 
                  ShiftNodes, (XtPointer) analysisd);
    XtAddCallback(analysisd->right, XtNcallback, 
                  ShiftNodes, (XtPointer) analysisd);

    base = 0;

    return analysisd;
}


/************************************************************************
 * Function:	AnalysisDialogPopup					*
 *									*
 * Description:	Pops up the specified material dialog.			*
 ************************************************************************/

void AnalysisDialogPopup (analysisd)
    AnalysisDialog analysisd;
{
    XtPopup (analysisd -> shell, XtGrabNone);
}
