# ifndef _ELEMENT_H
# define _ELEMENT_H
# include "fe.h"
# include "error.h"

# define trussType 0
extern struct definition trussDefinition;
extern int trussEltSetup ( );
extern int trussEltStress ( );

# define beamType 1
extern struct definition beamDefinition;
extern int beamEltSetup ( );
extern int beamEltStress ( );

# define beam3dType 2
extern struct definition beam3dDefinition;
extern int beam3dEltSetup ( );
extern int beam3dEltStress ( );

# define CSTPlaneStrainType 3
extern struct definition CSTPlaneStrainDefinition;
extern int CSTPlaneStrainEltSetup ( );
extern int CSTPlaneStrainEltStress ( );

# define CSTPlaneStressType 4
extern struct definition CSTPlaneStressDefinition;
extern int CSTPlaneStressEltSetup ( );
extern int CSTPlaneStressEltStress ( );

# define iso2d_PlaneStrainType 5
extern struct definition iso2d_PlaneStrainDefinition;
extern int iso2d_PlaneStrainEltSetup ( );
extern int iso2d_PlaneStrainEltStress ( );

# define iso2d_PlaneStressType 6
extern struct definition iso2d_PlaneStressDefinition;
extern int iso2d_PlaneStressEltSetup ( );
extern int iso2d_PlaneStressEltStress ( );

# define quad_PlaneStrainType 7
extern struct definition quad_PlaneStrainDefinition;
extern int quad_PlaneStrainEltSetup ( );
extern int quad_PlaneStrainEltStress ( );

# define quad_PlaneStressType 8
extern struct definition quad_PlaneStressDefinition;
extern int quad_PlaneStressEltSetup ( );
extern int quad_PlaneStressEltStress ( );

# define timoshenkoType 9
extern struct definition timoshenkoDefinition;
extern int timoshenkoEltSetup ( );
extern int timoshenkoEltStress ( );

# ifdef ELEMENTS
static struct {
    char      *name;
    int      (*setup) ( );
    int      (*stress) ( );
    Definition definition;
} ElementArray [ ] = {
    {"truss", trussEltSetup, trussEltStress, &trussDefinition},
    {"beam", beamEltSetup, beamEltStress, &beamDefinition},
    {"beam3d", beam3dEltSetup, beam3dEltStress, &beam3dDefinition},
    {"CSTPlaneStrain", CSTPlaneStrainEltSetup, CSTPlaneStrainEltStress, &CSTPlaneStrainDefinition},
    {"CSTPlaneStress", CSTPlaneStressEltSetup, CSTPlaneStressEltStress, &CSTPlaneStressDefinition},
    {"iso2d_PlaneStrain", iso2d_PlaneStrainEltSetup, iso2d_PlaneStrainEltStress, &iso2d_PlaneStrainDefinition},
    {"iso2d_PlaneStress", iso2d_PlaneStressEltSetup, iso2d_PlaneStressEltStress, &iso2d_PlaneStressDefinition},
    {"quad_PlaneStrain", quad_PlaneStrainEltSetup, quad_PlaneStrainEltStress, &quad_PlaneStrainDefinition},
    {"quad_PlaneStress", quad_PlaneStressEltSetup, quad_PlaneStressEltStress, &quad_PlaneStressDefinition},
    {"timoshenko", timoshenkoEltSetup, timoshenkoEltStress, &timoshenkoDefinition},
    {0, 0, 0, 0}
};
# endif

# endif /* _ELEMENT_H */
