/*
    This file is part of the FElt finite element analysis package.
    Copyright (C) 1993,1994 Jason I. Gobat and Darren C. Atkinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/************************************************************************
 * File:	matrix.h						*
 *									*
 * Description:	This file contains the public type, macro, and function	*
 *		declarations for the matrix package.			*
 ************************************************************************/

# ifndef _MATRIX_H
# define _MATRIX_H
# include <stdio.h>
# include "proto.h"

typedef struct matrix {
    unsigned nrows;
    unsigned ncols;
    double  *vdata;
    double **mdata;
} *Matrix, *Vector;

# define MatrixRows(m)	((m) -> nrows)
# define MatrixCols(m)	((m) -> ncols)
# define MatrixData(m)	((m) -> mdata)
# define VectorSize(v)	((v) -> nrows)
# define VectorData(v)	((v) -> vdata)

# define NullMatrix	((Matrix) 0)
# define NullVector	((Vector) 0)


extern Matrix CreateMatrix     PROTO ((unsigned, unsigned));
extern Matrix CreateSubmatrix  PROTO ((unsigned, unsigned));
extern Vector CreateVector     PROTO ((unsigned));
extern void   DestroyMatrix    PROTO ((Matrix));
extern void   DestroySubmatrix PROTO ((Matrix));
extern void   DestroyVector    PROTO ((Vector));
extern void   PrintMatrix      PROTO ((Matrix, FILE *));
extern void   PrintVector      PROTO ((Vector, FILE *));

extern Matrix SetSubmatrix     PROTO ((Matrix, Matrix, unsigned, unsigned));
extern Matrix ScaleMatrix      PROTO ((Matrix, Matrix, double, double));
extern Matrix AssignMatrix     PROTO ((Matrix, Matrix));
extern Matrix InvertMatrix     PROTO ((Matrix, Matrix));
extern Matrix TransposeMatrix  PROTO ((Matrix, Matrix));
extern Matrix MirrorMatrix     PROTO ((Matrix, Matrix));
extern Matrix ZeroMatrix       PROTO ((Matrix));

extern Matrix AddMatrices      PROTO ((Matrix, Matrix, Matrix));
extern Matrix SubtractMatrices PROTO ((Matrix, Matrix, Matrix));
extern Matrix MultiplyMatrices PROTO ((Matrix, Matrix, Matrix));

extern Matrix LUDecomposition  PROTO ((Matrix, Vector, Matrix));
extern Vector LUBackSolve      PROTO ((Vector, Matrix, Vector, Vector));
extern Vector EigenSolve       PROTO ((Vector, Matrix, Matrix));

# endif /* _MATRIX_H */

