#ifndef lint
static char yysccsid[] = "@(#)yaccpar	1.8 (Berkeley) 01/20/90";
#endif
#define YYBYACC 1
#line 2 "parser.y"
/*
    This file is part of the FElt finite element analysis package.
    Copyright (C) 1993,1994 Jason I. Gobat and Darren C. Atkinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/************************************************************************
 * File:	parser.y						*
 *									*
 * Description:	This file contains the yacc specification for the	*
 *		parser for the FElt 2.0 system.				*
 ************************************************************************/

# include <stdio.h>
# include "code.h"
# include "error.h"
# include "objects.h"
# include "problem.h"
# include "allocate.h"


/* Last parameters (default for some parameters is to inherit the last). */

static double		  last_x;		/* last x coordinate	   */
static double		  last_y;		/* last y coordinate	   */
static double		  last_z;		/* last z coordinate	   */
static char		 *last_constraint;	/* name of last constraint */
static char		 *last_material;	/* name of last material   */


/* Current objects (inherited attributes). */

static Item		  found;		/* current object	   */
static Node		  node;			/* current node		   */
static Element		  element;		/* current element	   */
static Material		  material;		/* current material	   */
static Distributed	  load;			/* current load		   */
static Force		  force;		/* current force	   */
static Constraint	  constraint;		/* current constraint	   */
static Definition	  definition;		/* current definition	   */


/* Dummy strucutures.  If an object is defined twice, the second definition
   is illegal and the current object is set to a dummy object. */

static struct node	  dummy_node;		/* dummy node		   */
static struct element	  dummy_element;	/* dummy element	   */
static struct material	  dummy_material;	/* dummy material	   */
static struct distributed dummy_load;		/* dummy distributed load  */
static struct force	  dummy_force;		/* dummy force		   */
static struct constraint  dummy_constraint;	/* dummy constraint	   */


/* Temporary arrays. */

static int		  int_array [1024];	/* temporary integer array */
static int		 *int_ptr;		/* pointer into array	   */
static Pair		  pair_array [1024];	/* temporary pair array	   */
static Pair		 *pair_ptr;		/* pointer into array	   */


/* Discrete expression variables. */

static int		  table_error = 0;	/* error indicator	   */
static double		  last_time = 0;	/* last time coordinate	   */
static double		 *table = NULL;		/* table of values	   */
static unsigned		  table_count = 0;	/* count of values	   */
static unsigned		  table_size = 0;	/* size of table	   */
#line 85 "parser.y"
typedef union {
    int       i;
    double    d;
    char     *s;
    Pair      p;
} YYSTYPE;
#line 95 "y.tab.c"
#define OR 257
#define AND 258
#define EQUALS 259
#define NEQUAL 260
#define LT_EQ 261
#define GT_EQ 262
#define LSHIFT 263
#define RSHIFT 264
#define UNARY 265
#define NAME 266
#define INTEGER 267
#define DOUBLE 268
#define TIME 269
#define SIN 270
#define COS 271
#define TAN 272
#define POW 273
#define EXP 274
#define LOG 275
#define LOG10 276
#define SQRT 277
#define HYPOT 278
#define FLOOR 279
#define CEIL 280
#define FMOD 281
#define FABS 282
#define ANALYSIS_TYPE 283
#define DIRECTION 284
#define CONSTRAINT 285
#define HINGED 286
#define NODE_DOF 287
#define MASS_MODE 288
#define PROBLEM 289
#define NODES 290
#define ELEMENTS 291
#define MATERIALS 292
#define LOADS 293
#define FORCES 294
#define CONSTRAINTS 295
#define ANALYSIS 296
#define END 297
#define TITLE_EQ 298
#define NODES_EQ 299
#define ELEMENTS_EQ 300
#define ANALYSIS_EQ 301
#define X_EQ 302
#define Y_EQ 303
#define Z_EQ 304
#define FORCE_EQ 305
#define CONSTRAINT_EQ 306
#define LOAD_EQ 307
#define MATERIAL_EQ 308
#define E_EQ 309
#define IX_EQ 310
#define IY_EQ 311
#define IZ_EQ 312
#define A_EQ 313
#define J_EQ 314
#define G_EQ 315
#define T_EQ 316
#define RHO_EQ 317
#define NU_EQ 318
#define KAPPA_EQ 319
#define RK_EQ 320
#define RM_EQ 321
#define DIRECTION_EQ 322
#define VALUES_EQ 323
#define FX_EQ 324
#define FY_EQ 325
#define FZ_EQ 326
#define MX_EQ 327
#define MY_EQ 328
#define MZ_EQ 329
#define TX_EQ 330
#define TY_EQ 331
#define TZ_EQ 332
#define RX_EQ 333
#define RY_EQ 334
#define RZ_EQ 335
#define VX_EQ 336
#define VY_EQ 337
#define VZ_EQ 338
#define AX_EQ 339
#define AY_EQ 340
#define AZ_EQ 341
#define ALPHA_EQ 342
#define BETA_EQ 343
#define GAMMA_EQ 344
#define DT_EQ 345
#define DURATION_EQ 346
#define DOFS_EQ 347
#define TABLE_EQ 348
#define PLOT_EQ 349
#define MASS_MODE_EQ 350
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,   12,   13,   13,   16,   16,   17,   17,   17,
   17,   17,   14,   14,   18,   18,   18,   18,   18,   18,
   18,   18,   19,   26,   26,   27,   28,   29,   29,   30,
   30,   30,   30,   30,   30,   20,   31,   32,   32,   33,
   34,   35,   35,   36,   36,   36,   36,   37,   37,   37,
    5,   38,   38,   38,   21,   39,   39,   40,   41,   42,
   42,   43,   43,   43,   43,   43,   43,   43,   43,   43,
   43,   43,   43,   43,   43,   22,   44,   44,   45,   46,
   47,   47,   48,   48,   48,   49,   49,   49,    1,    1,
   23,   50,   50,   51,   52,   53,   53,   54,   54,   54,
   54,   54,   54,   54,   24,   55,   55,   56,   57,   58,
   58,   59,   59,   59,   59,   59,   59,   59,   59,   59,
   59,   59,   59,   59,    2,    2,    3,    3,    3,   25,
   60,   60,   61,   61,   61,   61,   61,   61,   61,   61,
   61,   62,   62,   62,   63,   63,   63,   15,   15,   15,
   64,   64,   64,   65,    4,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,   10,   11,    8,    9,
};
short yylen[] = {                                         2,
    4,    3,    0,    2,    0,    2,    0,    2,    2,    2,
    2,    1,    2,    0,    1,    1,    1,    1,    1,    1,
    1,    1,    2,    2,    0,    2,    1,    2,    0,    2,
    2,    2,    2,    2,    1,    2,    1,    2,    0,    2,
    1,    2,    0,    4,    2,    2,    1,    3,    2,    1,
    1,    3,    2,    1,    2,    2,    0,    2,    1,    2,
    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    1,    2,    2,    0,    2,    1,
    2,    0,    2,    2,    1,    3,    2,    1,    5,    4,
    2,    2,    0,    2,    1,    2,    0,    2,    2,    2,
    2,    2,    2,    1,    2,    2,    0,    2,    1,    2,
    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    1,    1,    1,    1,    1,    1,    2,
    2,    0,    2,    2,    2,    2,    2,    4,    4,    2,
    1,    3,    2,    0,    3,    2,    0,    1,    1,    2,
    3,    2,    1,    5,    1,    7,    4,    4,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    2,    2,    2,    2,    3,    1,
    1,    1,    1,    4,    4,    4,    6,    4,    4,    4,
    4,    6,    4,    4,    6,    4,    0,    0,    0,    0,
};
short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,  180,  181,  182,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  183,    0,    0,  153,    7,
   14,    0,  175,  176,  177,  178,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  197,  199,  200,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    2,  150,    0,    0,  152,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  179,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  172,
  173,  174,  151,    0,   12,    0,    0,    0,    0,    6,
   25,   37,   57,   78,   93,  107,  132,    0,   13,   15,
   16,   17,   18,   19,   20,   21,   39,  184,  185,  186,
    0,  188,  189,  190,  191,    0,  193,  194,    0,  196,
    0,    0,    0,    0,    8,    9,   10,   11,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  154,  198,
   27,   24,   29,   59,   56,   61,   80,   77,   82,   95,
   92,   97,  109,  106,  111,  141,    0,    0,    0,    0,
    0,    0,    0,    0,  131,   41,   38,   43,  187,  192,
  195,    0,    0,    0,    0,    0,    0,  144,  133,  134,
  135,  136,  137,  147,  140,    0,    0,   35,    0,    0,
    0,    0,    0,   28,   75,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   60,   85,
    0,    0,   81,  104,    0,    0,    0,    0,    0,    0,
   96,  124,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  110,    0,    0,   47,    0,    0,
    0,   42,   30,   31,   32,   33,   34,   62,   63,   64,
   65,   66,   67,   68,   69,   70,   71,   72,   73,   74,
   83,    0,   88,    0,   98,   99,  100,  101,  102,  103,
  126,  112,  125,  113,  114,  128,  129,  115,  127,  116,
  117,  118,  119,  120,  121,  122,  123,  143,  138,    0,
  146,  139,    0,    0,   54,    0,   45,    0,    0,   87,
  142,  145,   51,   50,    0,   53,    0,    0,    0,   86,
   44,    0,   49,   52,    0,   90,   48,   89,
};
short yydgoto[] = {                                       2,
  293,  302,  308,  303,  334,  114,   26,   95,   96,   94,
  202,    3,   31,   77,   27,   76,  120,  129,  130,  131,
  132,  133,  134,  135,  136,  159,  172,  173,  203,  224,
  137,  165,  197,  198,  216,  272,  335,  326,  160,  175,
  176,  204,  239,  161,  178,  179,  205,  243,  294,  162,
  181,  182,  206,  251,  163,  184,  185,  207,  265,  164,
  195,  266,  267,   28,   29,
};
short yysindex[] = {                                   -288,
  538,    0, -272,  678,  678,  678,  678,    0,    0,    0,
  -16,  -13,   -1,    4,    9,   13,   18,   22,   31,   37,
   42,   44,   57,  678, 3487,    0, -241,    3,    0,    0,
    0,  678,    0,    0,    0,    0,  678,  678,  678,  678,
  678,  678,  678,  678,  678,  678,  678,  678,  678,   58,
  953,    0,    0,    0,  678,  678,  678,  678,  678,  678,
  678,  678,  678,  678,  678,  678,  678,  678,  678,  678,
    0,    0,   61,  678,    0, -187,  122,  953, 1265, 1358,
 1670, 1763, 1774, 2075, 2168, 3074, 3101, 3112, 3147, 3175,
 3186,  678,    0,  678,  678,  678,  453,  632,  465,   38,
   38,  -34,  -34,  -34,  -34,  -17,  -17,   -6,   -6,    0,
    0,    0,    0, 3487,    0, -157, -132, -131, -146,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  678,    0,    0,    0,    0,  678,    0,    0,  678,    0,
   97, 3215, 3306, 3498,    0,    0,    0,    0, -128, -124,
 -123, -121, -108, -251, -107, 3253, 3424, 3459,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   68,  678,  678,  678,
  678,  678,   70, -126,    0,    0,    0,    0,    0,    0,
    0,  678, -216,  292, -250, -221, -214,    0,    0,    0,
    0,    0,    0,    0,    0, -244, 3487,    0,  678,  678,
  678, -103,  -96,    0,    0,  678,  678,  678,  678,  678,
  678,  678,  678,  678,  678,  678,  678,  678,    0,    0,
 -113,  133,    0,    0,  538,  538,  538,  538,  538,  538,
    0,    0,  101,  101,  101,  366,  366,  366,  678,  678,
  678,  678,  678,  678,    0,  -26,  -42,    0,   86,  -88,
  -87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  -77,    0,  -25,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -75,
    0,    0,  -89,  -68,    0,  -40,    0,  -11,  133,    0,
    0,    0,    0,    0,  -23,    0,  -86,  678,  159,    0,
    0,  -68,    0,    0,  160,    0,    0,    0,
};
short yyrindex[] = {                                   -141,
    0,    0,  638,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -109,    0,    0,  227,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  177,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  671,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2285, 2156, 2061, 1656,
 1751,  846,  941, 1251, 1346,  441,  536,   16,  131,    0,
    0,    0,    0, 2690,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    1,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2472, 2380,    0,    0,    0,    0,  681,  715,
  749,  758,  766,  776,  788,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  621,  -44,  248,  329,  390,    0,    0,    0,
    0,    0,    0,    0,    0,  630, 2598,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  294,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  730,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,
};
short yygindex[] = {                                      0,
 -284, -217, -192,  -24, -319, 3037,    0,    0,    0,    0,
    0,    0,    0,    0, -117,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -21,
};
#define YYTABLESIZE 3762
short yytable[] = {                                      50,
    1,  323,   70,  337,  186,  240,   75,   68,   66,  330,
   67,  268,   69,    1,  292,  343,   30,  320,  329,   70,
  342,    6,  347,   37,   68,   66,   38,   67,   32,   69,
   70,    4,  338,    5,  244,   68,  304,  305,   39,  218,
   69,  252,   74,   40,  340,   72,   73,  187,   41,   50,
  322,  113,   42,  170,  269,   71,  170,   43,  170,  170,
  170,   44,  270,  271,  310,  311,  319,  151,  115,  341,
   45,  241,  242,  170,   70,  170,   46,  170,  170,   68,
   66,   47,   67,   48,   69,  219,  220,  221,  222,  223,
  188,  189,  190,  191,  192,  193,   49,   60,  194,   61,
   74,   92,  245,  246,  247,  248,  249,  250,  155,  170,
  116,  117,  118,  119,    7,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  295,  296,  297,
  298,  299,  300,    6,  156,  157,  158,  169,  171,  170,
   32,  174,  177,    4,  180,    5,  148,    3,    3,    3,
    3,    3,    3,    3,    3,    3,  148,  183,  208,  196,
  214,  215,  276,  209,  210,  211,  212,  213,  171,  277,
  291,  171,  292,  171,  171,  171,  324,  325,  327,  344,
  148,  148,  148,  148,  148,  148,  148,  148,  171,  328,
  171,  331,  171,  171,  273,  274,  275,  332,  333,  346,
  348,  278,  279,  280,  281,  282,  283,  284,  285,  286,
  287,  288,  289,  290,  148,  148,  148,  148,  148,  148,
  155,   58,    0,    0,  171,  336,    7,    0,   64,   65,
    0,  309,  309,  309,  312,  313,  314,  315,  316,  317,
  318,    0,    0,  333,  321,   58,   58,   58,   58,   58,
   58,   58,   58,    0,  171,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
   23,  170,  170,  170,  170,  170,  170,  170,  170,  170,
    0,  170,  170,    0,    0,    0,    0,    0,    0,    0,
   22,   22,   22,   22,   22,   22,   22,   22,   62,   63,
   64,   65,    0,  339,    0,  170,  170,  170,  170,  170,
  170,  170,  170,  345,  170,    0,    0,  170,  170,  170,
  170,  170,    0,    0,  170,  170,  170,  170,  170,  170,
  170,  170,  170,  170,  170,  170,  170,    0,    0,  170,
  170,  170,  170,  170,  170,  170,  170,  170,  170,  170,
  170,  170,  170,  170,  170,  170,  170,  170,  170,  170,
  170,  170,  170,    0,    0,  170,    0,    8,    9,   10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,    0,    0,  301,  171,  171,  171,  171,
  171,  171,  171,  171,  171,    0,  171,  171,    6,    0,
    0,    0,    0,    0,    0,   32,    0,    0,    4,    0,
    5,  121,  122,  123,  124,  125,  126,  127,  128,    0,
  171,  171,  171,  171,  171,  171,  171,  171,    0,  171,
    0,    0,  171,  171,  171,  171,  171,    0,    0,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,    0,    0,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  168,    0,
  171,  168,  149,    0,  168,    0,    0,    0,    0,   70,
   57,    7,  149,    0,   68,   66,    0,   67,  168,   69,
  168,   70,  168,  168,    0,    0,   68,   66,    0,   67,
    0,   69,   60,   79,   61,    0,  149,  149,  149,  149,
  149,  149,  149,  149,   60,    0,   61,    0,    0,    0,
    0,    0,    0,    0,  168,    0,    0,   79,   79,   79,
   79,   79,   79,   79,   79,    0,   56,  225,    0,   84,
  149,  149,  149,  149,  149,  149,    0,    0,    0,   84,
    0,    0,    0,    0,  168,    0,    0,    0,    0,    0,
    6,    0,    0,  169,    0,    0,  169,   24,    0,  169,
    4,    0,    5,   84,   84,   84,   84,   84,   84,   84,
   84,    0,    0,  169,   94,  169,    0,  169,  169,    0,
  226,  227,  228,  229,  230,  231,  232,  233,  234,  235,
  236,  237,  238,    0,    0,   84,   84,    0,   94,   94,
   94,   94,   94,   94,   94,   94,    0,    0,    0,  169,
    0,    0,    8,    9,   10,   11,   12,   13,   14,   15,
   16,   17,   18,   19,   20,   21,   22,   23,    0,    0,
  306,  307,    0,    0,    0,  108,    0,    0,    0,  169,
    0,    0,    0,    7,    0,    0,    0,    0,   70,   57,
    0,    0,    0,   68,   66,    0,   67,    0,   69,  108,
  108,  108,  108,  108,  108,  108,  108,    0,    0,    0,
    0,   60,    0,   61,    0,    0,  168,  168,  168,  168,
  168,  168,  168,  168,  168,    0,  168,  168,    0,    0,
    6,   58,   59,   62,   63,   64,   65,   32,    0,    0,
    4,    0,    5,   58,   59,   62,   63,   64,   65,    0,
  168,  168,  168,  168,  168,  168,  168,  168,    0,  168,
    0,    0,  168,  168,  168,  168,  168,    0,    0,  168,
  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,
  168,  168,    0,    0,  168,  168,  168,  168,  168,  168,
  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,
  168,  168,  168,  168,  168,  168,  168,  168,    0,    0,
  168,  169,  169,  169,  169,  169,  169,  169,  169,  169,
    0,  169,  169,    7,    8,    9,   10,   11,   12,   13,
   14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
    0,    0,    0,    0,    0,  169,  169,  169,  169,  169,
  169,  169,  169,    0,  169,    0,    0,  169,  169,  169,
  169,  169,    0,    0,  169,  169,  169,  169,  169,  169,
  169,  169,  169,  169,  169,  169,  169,    0,    0,  169,
  169,  169,  169,  169,  169,  169,  169,  169,  169,  169,
  169,  169,  169,  169,  169,  169,  169,  169,  169,  169,
  169,  169,  169,  164,    0,  169,  164,   26,    0,  164,
   58,   59,   62,   63,   64,   65,   40,    0,    0,    0,
    0,    0,    0,  164,    0,  164,    0,  164,  164,    0,
   26,   26,   26,   26,   26,   26,   26,   26,    0,   40,
   40,   40,   40,   40,   40,   40,   40,    5,    5,    5,
    5,    5,    5,    5,    5,    0,    0,    0,    0,  164,
    0,    0,    0,    0,    8,    9,   10,   11,   12,   13,
   14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
    4,    4,    4,    4,    4,    4,    4,    4,    0,  164,
   23,   23,   23,   23,   23,   23,   23,   23,  165,    0,
    0,  165,    0,    0,  165,   46,    0,    0,    0,   70,
   57,    0,    0,   93,   68,   66,   46,   67,  165,   69,
  165,    0,  165,  165,   55,   55,   55,   55,   55,   55,
   55,   55,   60,    0,   61,   52,    0,    0,    0,   46,
   46,   46,   46,   46,   46,   46,   46,    0,   46,    0,
    0,    0,    0,    0,  165,    0,   46,   46,   76,   76,
   76,   76,   76,   76,   76,   76,   56,   91,   91,   91,
   91,   91,   91,   91,   91,  105,  105,  105,  105,  105,
  105,  105,  105,    0,  165,  130,  130,  130,  130,  130,
  130,  130,  130,    0,    0,    0,   55,   36,   36,   36,
   36,   36,   36,   36,   36,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  164,  164,  164,  164,  164,  164,  164,    0,    0,
    0,  164,  164,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  164,  164,  164,  164,  164,
  164,  164,  164,    0,  164,    0,    0,  164,  164,  164,
  164,  164,    0,    0,  164,  164,  164,  164,  164,  164,
  164,  164,  164,  164,  164,  164,  164,    0,    0,  164,
  164,  164,  164,  164,  164,  164,  164,  164,  164,  164,
  164,  164,  164,  164,  164,  164,  164,  164,  164,  164,
  164,  164,  164,    0,    0,  164,  165,  165,  165,  165,
  165,  165,  165,    0,    0,    0,  165,  165,    0,   53,
   54,   58,   59,   62,   63,   64,   65,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  165,  165,  165,  165,  165,  165,  165,  165,    0,  165,
    0,    0,  165,  165,  165,  165,  165,    0,    0,  165,
  165,  165,  165,  165,  165,  165,  165,  165,  165,  165,
  165,  165,    0,    0,  165,  165,  165,  165,  165,  165,
  165,  165,  165,  165,  165,  165,  165,  165,  165,  165,
  165,  165,  165,  165,  165,  165,  165,  165,  166,    0,
  165,  166,    0,    0,  166,    0,    0,    0,    0,    0,
    0,   70,   57,    0,    0,  138,   68,   66,  166,   67,
  166,   69,  166,  166,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   60,    0,   61,   52,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  166,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   56,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  166,    0,    0,    0,    0,    0,
    0,    0,    0,  167,    0,    0,  167,    0,   55,  167,
    0,    0,    0,    0,   70,   57,    0,    0,  139,   68,
   66,    0,   67,  167,   69,  167,    0,  167,  167,    0,
    0,    0,    0,    0,    0,    0,    0,   60,    0,   61,
   52,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  167,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   56,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  167,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   55,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  166,  166,  166,  166,
  166,  166,  166,    0,    0,    0,  166,  166,    0,    0,
    0,   53,   54,   58,   59,   62,   63,   64,   65,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  166,  166,  166,  166,  166,  166,  166,  166,    0,  166,
    0,    0,  166,  166,  166,  166,  166,    0,    0,  166,
  166,  166,  166,  166,  166,  166,  166,  166,  166,  166,
  166,  166,    0,    0,  166,  166,  166,  166,  166,  166,
  166,  166,  166,  166,  166,  166,  166,  166,  166,  166,
  166,  166,  166,  166,  166,  166,  166,  166,    0,    0,
  166,  167,  167,  167,  167,  167,  167,  167,    0,    0,
    0,  167,  167,    0,   53,   54,   58,   59,   62,   63,
   64,   65,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  167,  167,  167,  167,  167,
  167,  167,  167,    0,  167,    0,    0,  167,  167,  167,
  167,  167,    0,    0,  167,  167,  167,  167,  167,  167,
  167,  167,  167,  167,  167,  167,  167,    0,    0,  167,
  167,  167,  167,  167,  167,  167,  167,  167,  167,  167,
  167,  167,  167,  167,  167,  167,  167,  167,  167,  167,
  167,  167,  167,  162,    0,  167,  162,    0,    0,  162,
    0,    0,    0,    0,    0,    0,   70,   57,    0,    0,
  140,   68,   66,  162,   67,    0,   69,    0,  162,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
    0,   61,   52,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   56,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
    0,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  163,    0,   55,  163,    0,    0,    0,    0,   70,
   57,    0,    0,    0,   68,   66,  141,   67,  163,   69,
   70,   57,    0,  163,  142,   68,   66,    0,   67,    0,
   69,    0,   60,    0,   61,   52,    0,    0,    0,    0,
    0,    0,    0,   60,    0,   61,   52,    0,    0,    0,
    0,    0,    0,    0,  163,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   56,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   56,    0,    0,
    0,    0,    0,    0,  163,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   55,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  162,  162,  162,  162,    0,    0,    0,    0,
    0,  162,  162,    0,    0,    0,   53,   54,   58,   59,
   62,   63,   64,   65,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  162,  162,  162,  162,
  162,  162,  162,    0,  162,    0,    0,  162,  162,  162,
  162,  162,    0,    0,  162,  162,  162,  162,  162,  162,
  162,  162,  162,  162,  162,  162,  162,    0,    0,  162,
  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,
  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,
  162,  162,  162,    0,    0,  162,  163,  163,  163,  163,
  163,    0,    0,    0,    0,    0,  163,  163,    0,   53,
   54,   58,   59,   62,   63,   64,   65,    0,    0,    0,
   53,   54,   58,   59,   62,   63,   64,   65,    0,    0,
  163,  163,  163,  163,  163,  163,  163,  163,    0,  163,
    0,    0,  163,  163,  163,  163,  163,    0,    0,  163,
  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,
  163,  163,    0,    0,  163,  163,  163,  163,  163,  163,
  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,
  163,  163,  163,  163,  163,  163,  163,  163,  161,    0,
  163,  161,    0,    0,  161,    0,    0,    0,    0,    0,
    0,   70,   57,    0,    0,  143,   68,   66,  161,   67,
    0,   69,    0,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   60,    0,   61,   52,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  161,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   56,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  161,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  160,    0,   55,  160,
    0,    0,    0,    0,   70,   57,    0,    0,  144,   68,
   66,    0,   67,  160,   69,    0,    0,    0,  160,    0,
    0,    0,    0,    0,    0,    0,    0,   60,    0,   61,
   52,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  160,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   56,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  160,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   55,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  161,  161,  161,    0,
    0,    0,    0,    0,    0,  159,  161,  161,  159,    0,
    0,   53,   54,   58,   59,   62,   63,   64,   65,    0,
    0,    0,  159,    0,    0,    0,    0,  159,    0,    0,
  161,  161,  161,  161,  161,  161,  161,  161,    0,  161,
    0,    0,  161,  161,  161,  161,  161,    0,    0,  161,
  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,
  161,  161,    0,    0,  161,  161,  161,  161,  161,  161,
  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,
  161,  161,  161,  161,  161,  161,  161,  161,  159,    0,
  161,  160,  160,  160,    0,    0,    0,    0,    0,    0,
  158,  160,  160,  158,   53,   54,   58,   59,   62,   63,
   64,   65,    0,    0,    0,    0,    0,  158,    0,    0,
    0,    0,  158,    0,    0,  160,  160,  160,  160,  160,
  160,  160,  160,    0,  160,    0,    0,  160,  160,  160,
  160,  160,    0,    0,  160,  160,  160,  160,  160,  160,
  160,  160,  160,  160,  160,  160,  160,    0,    0,  160,
  160,  160,  160,  160,  160,  160,  160,  160,  160,  160,
  160,  160,  160,  160,  160,  160,  160,  160,  160,  160,
  160,  160,  160,    0,    0,  160,    0,    0,    0,    0,
    0,    0,  157,    0,    0,  157,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,
    0,    0,    0,    0,  157,    0,    0,    0,    0,    0,
  159,  159,  159,    0,    0,    0,    0,    0,    0,    0,
  159,  159,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  159,  159,  159,  159,  159,  159,
  159,  159,    0,  159,    0,    0,  159,  159,  159,  159,
  159,    0,    0,  159,  159,  159,  159,  159,  159,  159,
  159,  159,  159,  159,  159,  159,    0,    0,  159,  159,
  159,  159,  159,  159,  159,  159,  159,  159,  159,  159,
  159,  159,  159,  159,  159,  159,  159,  159,  159,  159,
  159,  159,    0,    0,  159,  158,  158,  158,  156,    0,
    0,  156,    0,    0,    0,  158,  158,    0,    0,    0,
    0,    0,    0,    0,    0,  156,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  158,
  158,  158,  158,  158,  158,  158,  158,    0,  158,    0,
    0,  158,  158,  158,  158,  158,    0,    0,  158,  158,
  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,
  158,    0,    0,  158,  158,  158,  158,  158,  158,  158,
  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,
  158,  158,  158,  158,  158,  158,  158,  157,  157,  158,
  155,    0,    0,  155,    0,    0,    0,  157,  157,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  157,  157,  157,  157,  157,  157,  157,  157,    0,
  157,    0,    0,  157,  157,  157,  157,  157,    0,    0,
  157,  157,  157,  157,  157,  157,  157,  157,  157,  157,
  157,  157,  157,    0,    0,  157,  157,  157,  157,  157,
  157,  157,  157,  157,  157,  157,  157,  157,  157,  157,
  157,  157,  157,  157,  157,  157,  157,  157,  157,    0,
    0,  157,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  156,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  156,  156,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  156,  156,  156,
  156,  156,  156,  156,  156,    0,  156,    0,    0,  156,
  156,  156,  156,  156,    0,    0,  156,  156,  156,  156,
  156,  156,  156,  156,  156,  156,  156,  156,  156,    0,
    0,  156,  156,  156,  156,  156,  156,  156,  156,  156,
  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,
  156,  156,  156,  156,  156,  155,    0,  156,    0,    0,
    0,    0,    0,    0,    0,  155,  155,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  155,  155,  155,  155,  155,  155,  155,    0,  155,    0,
    0,  155,  155,  155,  155,  155,    0,    0,  155,  155,
  155,  155,  155,  155,  155,  155,  155,  155,  155,  155,
  155,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  155,  155,  155,  155,  155,  155,  155,  155,  155,  155,
  155,  155,  155,  155,  155,  155,  155,   25,    0,  155,
   33,   34,   35,   36,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   51,    0,    0,    0,    0,    0,    0,    0,   78,    0,
    0,    0,    0,   79,   80,   81,   82,   83,   84,   85,
   86,   87,   88,   89,   90,   91,    0,    0,    0,    0,
    0,   97,   98,   99,  100,  101,  102,  103,  104,  105,
  106,  107,  108,  109,  110,  111,  112,    0,    0,    0,
   70,   57,    0,    0,  145,   68,   66,    0,   67,    0,
   69,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  152,  153,  154,   60,    0,   61,   52,   70,   57,    0,
    0,    0,   68,   66,  146,   67,    0,   69,   70,   57,
    0,    0,  147,   68,   66,    0,   67,    0,   69,    0,
   60,    0,   61,   52,    0,    0,    0,   56,    0,    0,
    0,   60,    0,   61,   52,    0,    0,  166,    0,    0,
    0,    0,  167,   70,   57,  168,    0,  148,   68,   66,
    0,   67,    0,   69,   56,    0,    0,   55,    0,    0,
    0,    0,    0,    0,    0,   56,   60,    0,   61,   52,
    0,   70,   57,    0,    0,    0,   68,   66,  149,   67,
    0,   69,   70,   57,   55,    0,  150,   68,   66,    0,
   67,    0,   69,    0,   60,   55,   61,   52,  217,    0,
   56,    0,    0,    0,    0,   60,    0,   61,   52,    0,
    0,   70,   57,    0,    0,    0,   68,   66,    0,   67,
    0,   69,    0,    0,    0,    0,    0,    0,   56,    0,
   55,    0,  170,    0,   60,    0,   61,   52,    0,   56,
    0,   25,   25,   25,   25,   25,   25,    0,    0,   70,
   57,    0,    0,  199,   68,   66,    0,   67,   55,   69,
    0,    0,    0,    0,    0,    0,    0,    0,   56,   55,
    0,    0,   60,    0,   61,   52,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   53,   54,   58,   59,   62,   63,   64,   65,   55,    0,
    0,    0,   70,   57,    0,    0,   56,   68,   66,    0,
   67,    0,   69,    0,    0,    0,    0,   53,   54,   58,
   59,   62,   63,   64,   65,   60,    0,   61,   53,   54,
   58,   59,   62,   63,   64,   65,   55,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   56,
    0,    0,    0,   53,   54,   58,   59,   62,   63,   64,
   65,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   55,
    0,   53,   54,   58,   59,   62,   63,   64,   65,    0,
    0,    0,   53,   54,   58,   59,   62,   63,   64,   65,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   70,   57,    0,    0,  200,   68,   66,    0,   67,    0,
   69,   53,   54,   58,   59,   62,   63,   64,   65,    0,
    0,    0,    0,   60,    0,   61,   52,    0,    0,    0,
    0,    0,    0,    0,    0,   70,   57,    0,    0,  201,
   68,   66,    0,   67,    0,   69,    0,    0,    0,   53,
   54,   58,   59,   62,   63,   64,   65,   56,   60,    0,
   61,   52,    0,   70,   57,    0,    0,    0,   68,   66,
    0,   67,    0,   69,   70,   57,    0,    0,    0,   68,
   66,    0,   67,    0,   69,    0,   60,   55,   61,   52,
    0,    0,   56,    0,    0,    0,    0,   60,    0,   61,
    0,    0,    0,   54,   58,   59,   62,   63,   64,   65,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   56,    0,   55,    0,    0,    0,    0,    0,    0,    0,
    0,   56,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   55,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   55,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   53,   54,   58,   59,   62,   63,   64,   65,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   53,   54,   58,   59,   62,
   63,   64,   65,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   53,   54,   58,   59,   62,   63,   64,
   65,    0,    0,    0,    0,    0,   58,   59,   62,   63,
   64,   65,
};
short yycheck[] = {                                      24,
    0,   44,   37,   44,  256,  256,   28,   42,   43,  294,
   45,  256,   47,  302,   40,  335,  289,   44,   44,   37,
   44,   33,  342,   40,   42,   43,   40,   45,   40,   47,
   37,   43,   44,   45,  256,   42,  254,  255,   40,  256,
   47,  256,   40,   40,  329,   43,   44,  299,   40,   74,
   93,   73,   40,   38,  299,  297,   41,   40,   43,   44,
   45,   40,  307,  308,  257,  258,   93,   92,  256,   93,
   40,  322,  323,   58,   37,   60,   40,   62,   63,   42,
   43,   40,   45,   40,   47,  302,  303,  304,  305,  306,
  342,  343,  344,  345,  346,  347,   40,   60,  350,   62,
   40,   44,  324,  325,  326,  327,  328,  329,  266,   94,
  298,  299,  300,  301,  126,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  245,  246,  247,
  248,  249,  250,   33,  267,  267,  283,   41,  267,  124,
   40,  266,  266,   43,  266,   45,  256,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  266,  266,   91,  267,
   91,  288,  266,  188,  189,  190,  191,  192,   38,  266,
  284,   41,   40,   43,   44,   45,   91,  266,  266,  266,
  290,  291,  292,  293,  294,  295,  296,  297,   58,  267,
   60,  267,   62,   63,  219,  220,  221,  287,  267,   41,
   41,  226,  227,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  324,  325,  326,  327,  328,  329,
   44,  266,   -1,   -1,   94,  266,  126,   -1,  263,  264,
   -1,  256,  257,  258,  259,  260,  261,  262,  263,  264,
  267,   -1,   -1,  267,  287,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  124,  267,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,  278,  279,  280,  281,
  282,  256,  257,  258,  259,  260,  261,  262,  263,  264,
   -1,  266,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,  261,  262,
  263,  264,   -1,  328,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,  338,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   -1,   -1,  350,   -1,  267,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,  278,  279,
  280,  281,  282,   -1,   -1,  285,  256,  257,  258,  259,
  260,  261,  262,  263,  264,   -1,  266,  267,   33,   -1,
   -1,   -1,   -1,   -1,   -1,   40,   -1,   -1,   43,   -1,
   45,  290,  291,  292,  293,  294,  295,  296,  297,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,   38,   -1,
  350,   41,  256,   -1,   44,   -1,   -1,   -1,   -1,   37,
   38,  126,  266,   -1,   42,   43,   -1,   45,   58,   47,
   60,   37,   62,   63,   -1,   -1,   42,   43,   -1,   45,
   -1,   47,   60,  266,   62,   -1,  290,  291,  292,  293,
  294,  295,  296,  297,   60,   -1,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   94,   -1,   -1,  290,  291,  292,
  293,  294,  295,  296,  297,   -1,   94,  256,   -1,  256,
  324,  325,  326,  327,  328,  329,   -1,   -1,   -1,  266,
   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,   -1,
   33,   -1,   -1,   38,   -1,   -1,   41,   40,   -1,   44,
   43,   -1,   45,  290,  291,  292,  293,  294,  295,  296,
  297,   -1,   -1,   58,  266,   60,   -1,   62,   63,   -1,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
  319,  320,  321,   -1,   -1,  322,  323,   -1,  290,  291,
  292,  293,  294,  295,  296,  297,   -1,   -1,   -1,   94,
   -1,   -1,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,  278,  279,  280,  281,  282,   -1,   -1,
  285,  286,   -1,   -1,   -1,  266,   -1,   -1,   -1,  124,
   -1,   -1,   -1,  126,   -1,   -1,   -1,   -1,   37,   38,
   -1,   -1,   -1,   42,   43,   -1,   45,   -1,   47,  290,
  291,  292,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   60,   -1,   62,   -1,   -1,  256,  257,  258,  259,
  260,  261,  262,  263,  264,   -1,  266,  267,   -1,   -1,
   33,  259,  260,  261,  262,  263,  264,   40,   -1,   -1,
   43,   -1,   45,  259,  260,  261,  262,  263,  264,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,   -1,   -1,
  350,  256,  257,  258,  259,  260,  261,  262,  263,  264,
   -1,  266,  267,  126,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,  278,  279,  280,  281,  282,
   -1,   -1,   -1,   -1,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   38,   -1,  350,   41,  267,   -1,   44,
  259,  260,  261,  262,  263,  264,  267,   -1,   -1,   -1,
   -1,   -1,   -1,   58,   -1,   60,   -1,   62,   63,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  290,
  291,  292,  293,  294,  295,  296,  297,  290,  291,  292,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,   -1,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,  278,  279,  280,  281,  282,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  124,
  290,  291,  292,  293,  294,  295,  296,  297,   38,   -1,
   -1,   41,   -1,   -1,   44,  256,   -1,   -1,   -1,   37,
   38,   -1,   -1,   41,   42,   43,  267,   45,   58,   47,
   60,   -1,   62,   63,  290,  291,  292,  293,  294,  295,
  296,  297,   60,   -1,   62,   63,   -1,   -1,   -1,  290,
  291,  292,  293,  294,  295,  296,  297,   -1,  299,   -1,
   -1,   -1,   -1,   -1,   94,   -1,  307,  308,  290,  291,
  292,  293,  294,  295,  296,  297,   94,  290,  291,  292,
  293,  294,  295,  296,  297,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  124,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,   -1,   -1,  124,  290,  291,  292,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  256,  257,  258,  259,  260,  261,  262,   -1,   -1,
   -1,  266,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   -1,   -1,  350,  256,  257,  258,  259,
  260,  261,  262,   -1,   -1,   -1,  266,  267,   -1,  257,
  258,  259,  260,  261,  262,  263,  264,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,   38,   -1,
  350,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,
   -1,   37,   38,   -1,   -1,   41,   42,   43,   58,   45,
   60,   47,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   -1,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   94,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   38,   -1,   -1,   41,   -1,  124,   44,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   41,   42,
   43,   -1,   45,   58,   47,   60,   -1,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,  259,
  260,  261,  262,   -1,   -1,   -1,  266,  267,   -1,   -1,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,   -1,   -1,
  350,  256,  257,  258,  259,  260,  261,  262,   -1,   -1,
   -1,  266,  267,   -1,  257,  258,  259,  260,  261,  262,
  263,  264,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   38,   -1,  350,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,
   41,   42,   43,   58,   45,   -1,   47,   -1,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   38,   -1,
   -1,   41,   -1,  124,   44,   -1,   -1,   -1,   -1,   37,
   38,   -1,   -1,   -1,   42,   43,   44,   45,   58,   47,
   37,   38,   -1,   63,   41,   42,   43,   -1,   45,   -1,
   47,   -1,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   60,   -1,   62,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   94,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,   -1,
   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  256,  257,  258,  259,  260,   -1,   -1,   -1,   -1,
   -1,  266,  267,   -1,   -1,   -1,  257,  258,  259,  260,
  261,  262,  263,  264,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   -1,   -1,  350,  256,  257,  258,  259,
  260,   -1,   -1,   -1,   -1,   -1,  266,  267,   -1,  257,
  258,  259,  260,  261,  262,  263,  264,   -1,   -1,   -1,
  257,  258,  259,  260,  261,  262,  263,  264,   -1,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,   38,   -1,
  350,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,
   -1,   37,   38,   -1,   -1,   41,   42,   43,   58,   45,
   -1,   47,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   -1,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   94,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   41,   -1,  124,   44,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   41,   42,
   43,   -1,   45,   58,   47,   -1,   -1,   -1,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,   -1,
   -1,   -1,   -1,   -1,   -1,   41,  266,  267,   44,   -1,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,   -1,
   -1,   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,
  290,  291,  292,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,   -1,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,  334,  335,  336,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,  124,   -1,
  350,  256,  257,  258,   -1,   -1,   -1,   -1,   -1,   -1,
   41,  266,  267,   44,  257,  258,  259,  260,  261,  262,
  263,  264,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,
   -1,   -1,   63,   -1,   -1,  290,  291,  292,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,  321,   -1,   -1,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,   -1,   -1,  350,   -1,   -1,   -1,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,
  256,  257,  258,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  266,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,  292,  293,  294,  295,
  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,  310,  311,  312,  313,  314,  315,
  316,  317,  318,  319,  320,  321,   -1,   -1,  324,  325,
  326,  327,  328,  329,  330,  331,  332,  333,  334,  335,
  336,  337,  338,  339,  340,  341,  342,  343,  344,  345,
  346,  347,   -1,   -1,  350,  256,  257,  258,   41,   -1,
   -1,   44,   -1,   -1,   -1,  266,  267,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,  292,  293,  294,  295,  296,  297,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,  319,  320,
  321,   -1,   -1,  324,  325,  326,  327,  328,  329,  330,
  331,  332,  333,  334,  335,  336,  337,  338,  339,  340,
  341,  342,  343,  344,  345,  346,  347,  256,  257,  350,
   41,   -1,   -1,   44,   -1,   -1,   -1,  266,  267,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,  292,  293,  294,  295,  296,  297,   -1,
  299,   -1,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
  319,  320,  321,   -1,   -1,  324,  325,  326,  327,  328,
  329,  330,  331,  332,  333,  334,  335,  336,  337,  338,
  339,  340,  341,  342,  343,  344,  345,  346,  347,   -1,
   -1,  350,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  256,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  266,  267,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,  292,
  293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,  319,  320,  321,   -1,
   -1,  324,  325,  326,  327,  328,  329,  330,  331,  332,
  333,  334,  335,  336,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  256,   -1,  350,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  266,  267,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,  292,  293,  294,  295,  296,  297,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,  319,  320,
  321,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  330,
  331,  332,  333,  334,  335,  336,  337,  338,  339,  340,
  341,  342,  343,  344,  345,  346,  347,    1,   -1,  350,
    4,    5,    6,    7,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   24,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   32,   -1,
   -1,   -1,   -1,   37,   38,   39,   40,   41,   42,   43,
   44,   45,   46,   47,   48,   49,   -1,   -1,   -1,   -1,
   -1,   55,   56,   57,   58,   59,   60,   61,   62,   63,
   64,   65,   66,   67,   68,   69,   70,   -1,   -1,   -1,
   37,   38,   -1,   -1,   41,   42,   43,   -1,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   94,   95,   96,   60,   -1,   62,   63,   37,   38,   -1,
   -1,   -1,   42,   43,   44,   45,   -1,   47,   37,   38,
   -1,   -1,   41,   42,   43,   -1,   45,   -1,   47,   -1,
   60,   -1,   62,   63,   -1,   -1,   -1,   94,   -1,   -1,
   -1,   60,   -1,   62,   63,   -1,   -1,  141,   -1,   -1,
   -1,   -1,  146,   37,   38,  149,   -1,   41,   42,   43,
   -1,   45,   -1,   47,   94,   -1,   -1,  124,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   94,   60,   -1,   62,   63,
   -1,   37,   38,   -1,   -1,   -1,   42,   43,   44,   45,
   -1,   47,   37,   38,  124,   -1,   41,   42,   43,   -1,
   45,   -1,   47,   -1,   60,  124,   62,   63,  202,   -1,
   94,   -1,   -1,   -1,   -1,   60,   -1,   62,   63,   -1,
   -1,   37,   38,   -1,   -1,   -1,   42,   43,   -1,   45,
   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
  124,   -1,   58,   -1,   60,   -1,   62,   63,   -1,   94,
   -1,  245,  246,  247,  248,  249,  250,   -1,   -1,   37,
   38,   -1,   -1,   41,   42,   43,   -1,   45,  124,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,  124,
   -1,   -1,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,  258,  259,  260,  261,  262,  263,  264,  124,   -1,
   -1,   -1,   37,   38,   -1,   -1,   94,   42,   43,   -1,
   45,   -1,   47,   -1,   -1,   -1,   -1,  257,  258,  259,
  260,  261,  262,  263,  264,   60,   -1,   62,  257,  258,
  259,  260,  261,  262,  263,  264,  124,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,  257,  258,  259,  260,  261,  262,  263,
  264,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,   -1,
   -1,   -1,  257,  258,  259,  260,  261,  262,  263,  264,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   37,   38,   -1,   -1,   41,   42,   43,   -1,   45,   -1,
   47,  257,  258,  259,  260,  261,  262,  263,  264,   -1,
   -1,   -1,   -1,   60,   -1,   62,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   41,
   42,   43,   -1,   45,   -1,   47,   -1,   -1,   -1,  257,
  258,  259,  260,  261,  262,  263,  264,   94,   60,   -1,
   62,   63,   -1,   37,   38,   -1,   -1,   -1,   42,   43,
   -1,   45,   -1,   47,   37,   38,   -1,   -1,   -1,   42,
   43,   -1,   45,   -1,   47,   -1,   60,  124,   62,   63,
   -1,   -1,   94,   -1,   -1,   -1,   -1,   60,   -1,   62,
   -1,   -1,   -1,  258,  259,  260,  261,  262,  263,  264,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   94,   -1,  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,  258,  259,  260,  261,  262,  263,  264,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,  258,  259,  260,  261,
  262,  263,  264,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,  258,  259,  260,  261,  262,  263,
  264,   -1,   -1,   -1,   -1,   -1,  259,  260,  261,  262,
  263,  264,
};
#define YYFINAL 2
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 350
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"'!'",0,0,0,"'%'","'&'",0,"'('","')'","'*'","'+'","','","'-'",0,"'/'",0,0,0,0,0,
0,0,0,0,0,"':'",0,"'<'",0,"'>'","'?'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"'['",0,"']'","'^'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,"'|'",0,"'~'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"OR","AND","EQUALS","NEQUAL",
"LT_EQ","GT_EQ","LSHIFT","RSHIFT","UNARY","NAME","INTEGER","DOUBLE","TIME",
"SIN","COS","TAN","POW","EXP","LOG","LOG10","SQRT","HYPOT","FLOOR","CEIL",
"FMOD","FABS","ANALYSIS_TYPE","DIRECTION","CONSTRAINT","HINGED","NODE_DOF",
"MASS_MODE","PROBLEM","NODES","ELEMENTS","MATERIALS","LOADS","FORCES",
"CONSTRAINTS","ANALYSIS","END","TITLE_EQ","NODES_EQ","ELEMENTS_EQ",
"ANALYSIS_EQ","X_EQ","Y_EQ","Z_EQ","FORCE_EQ","CONSTRAINT_EQ","LOAD_EQ",
"MATERIAL_EQ","E_EQ","IX_EQ","IY_EQ","IZ_EQ","A_EQ","J_EQ","G_EQ","T_EQ",
"RHO_EQ","NU_EQ","KAPPA_EQ","RK_EQ","RM_EQ","DIRECTION_EQ","VALUES_EQ","FX_EQ",
"FY_EQ","FZ_EQ","MX_EQ","MY_EQ","MZ_EQ","TX_EQ","TY_EQ","TZ_EQ","RX_EQ","RY_EQ",
"RZ_EQ","VX_EQ","VY_EQ","VZ_EQ","AX_EQ","AY_EQ","AZ_EQ","ALPHA_EQ","BETA_EQ",
"GAMMA_EQ","DT_EQ","DURATION_EQ","DOFS_EQ","TABLE_EQ","PLOT_EQ","MASS_MODE_EQ",
};
char *yyrule[] = {
"$accept : specification",
"specification : initialize problem_description section_list END",
"specification : X_EQ variable_expression END",
"initialize :",
"problem_description : PROBLEM problem_parameter_list",
"problem_description :",
"problem_parameter_list : problem_parameter_list problem_parameter",
"problem_parameter_list :",
"problem_parameter : TITLE_EQ NAME",
"problem_parameter : NODES_EQ INTEGER",
"problem_parameter : ELEMENTS_EQ INTEGER",
"problem_parameter : ANALYSIS_EQ ANALYSIS_TYPE",
"problem_parameter : error",
"section_list : section_list section",
"section_list :",
"section : node_section",
"section : element_section",
"section : material_section",
"section : load_section",
"section : force_section",
"section : constraint_section",
"section : analysis_section",
"section : END",
"node_section : NODES node_definition_list",
"node_definition_list : node_definition_list node_definition",
"node_definition_list :",
"node_definition : node_number node_parameter_list",
"node_number : INTEGER",
"node_parameter_list : node_parameter_list node_parameter",
"node_parameter_list :",
"node_parameter : X_EQ constant_expression",
"node_parameter : Y_EQ constant_expression",
"node_parameter : Z_EQ constant_expression",
"node_parameter : FORCE_EQ NAME",
"node_parameter : CONSTRAINT_EQ NAME",
"node_parameter : error",
"element_section : element_header element_definition_list",
"element_header : ELEMENTS",
"element_definition_list : element_definition_list element_definition",
"element_definition_list :",
"element_definition : element_number element_parameter_list",
"element_number : INTEGER",
"element_parameter_list : element_parameter_list element_parameter",
"element_parameter_list :",
"element_parameter : NODES_EQ '[' element_node_list ']'",
"element_parameter : MATERIAL_EQ NAME",
"element_parameter : LOAD_EQ element_load_list",
"element_parameter : error",
"element_node_list : element_node_list ',' element_node",
"element_node_list : element_node_list element_node",
"element_node_list : element_node",
"element_node : INTEGER",
"element_load_list : element_load_list ',' NAME",
"element_load_list : element_load_list NAME",
"element_load_list : NAME",
"material_section : MATERIALS material_definition_list",
"material_definition_list : material_definition_list material_definition",
"material_definition_list :",
"material_definition : material_name material_parameter_list",
"material_name : NAME",
"material_parameter_list : material_parameter_list material_parameter",
"material_parameter_list :",
"material_parameter : E_EQ constant_expression",
"material_parameter : IX_EQ constant_expression",
"material_parameter : IY_EQ constant_expression",
"material_parameter : IZ_EQ constant_expression",
"material_parameter : A_EQ constant_expression",
"material_parameter : J_EQ constant_expression",
"material_parameter : G_EQ constant_expression",
"material_parameter : T_EQ constant_expression",
"material_parameter : RHO_EQ constant_expression",
"material_parameter : NU_EQ constant_expression",
"material_parameter : KAPPA_EQ constant_expression",
"material_parameter : RK_EQ constant_expression",
"material_parameter : RM_EQ constant_expression",
"material_parameter : error",
"load_section : LOADS load_definition_list",
"load_definition_list : load_definition_list load_definition",
"load_definition_list :",
"load_definition : load_name load_parameter_list",
"load_name : NAME",
"load_parameter_list : load_parameter_list load_parameter",
"load_parameter_list :",
"load_parameter : DIRECTION_EQ DIRECTION",
"load_parameter : VALUES_EQ value_pair_list",
"load_parameter : error",
"value_pair_list : value_pair_list ',' value_pair",
"value_pair_list : value_pair_list value_pair",
"value_pair_list : value_pair",
"value_pair : '(' INTEGER ',' constant_expression ')'",
"value_pair : '(' INTEGER constant_expression ')'",
"force_section : FORCES force_definition_list",
"force_definition_list : force_definition_list force_definition",
"force_definition_list :",
"force_definition : force_name force_parameter_list",
"force_name : NAME",
"force_parameter_list : force_parameter_list force_parameter",
"force_parameter_list :",
"force_parameter : FX_EQ variable_expression",
"force_parameter : FY_EQ variable_expression",
"force_parameter : FZ_EQ variable_expression",
"force_parameter : MX_EQ variable_expression",
"force_parameter : MY_EQ variable_expression",
"force_parameter : MZ_EQ variable_expression",
"force_parameter : error",
"constraint_section : CONSTRAINTS constraint_definition_list",
"constraint_definition_list : constraint_definition_list constraint_definition",
"constraint_definition_list :",
"constraint_definition : constraint_name constraint_parameter_list",
"constraint_name : NAME",
"constraint_parameter_list : constraint_parameter_list constraint_parameter",
"constraint_parameter_list :",
"constraint_parameter : TX_EQ translation",
"constraint_parameter : TY_EQ translation",
"constraint_parameter : TZ_EQ translation",
"constraint_parameter : RX_EQ rotation",
"constraint_parameter : RY_EQ rotation",
"constraint_parameter : RZ_EQ rotation",
"constraint_parameter : VX_EQ constant_expression",
"constraint_parameter : VY_EQ constant_expression",
"constraint_parameter : VZ_EQ constant_expression",
"constraint_parameter : AX_EQ constant_expression",
"constraint_parameter : AY_EQ constant_expression",
"constraint_parameter : AZ_EQ constant_expression",
"constraint_parameter : error",
"translation : constant_expression",
"translation : CONSTRAINT",
"rotation : constant_expression",
"rotation : CONSTRAINT",
"rotation : HINGED",
"analysis_section : ANALYSIS analysis_parameter_list",
"analysis_parameter_list : analysis_parameter_list analysis_parameter",
"analysis_parameter_list :",
"analysis_parameter : ALPHA_EQ constant_expression",
"analysis_parameter : BETA_EQ constant_expression",
"analysis_parameter : GAMMA_EQ constant_expression",
"analysis_parameter : DT_EQ constant_expression",
"analysis_parameter : DURATION_EQ constant_expression",
"analysis_parameter : NODES_EQ '[' analysis_node_list ']'",
"analysis_parameter : DOFS_EQ '[' analysis_dof_list ']'",
"analysis_parameter : MASS_MODE_EQ MASS_MODE",
"analysis_parameter : error",
"analysis_node_list : analysis_node_list ',' INTEGER",
"analysis_node_list : analysis_node_list INTEGER",
"analysis_node_list :",
"analysis_dof_list : analysis_dof_list ',' NODE_DOF",
"analysis_dof_list : analysis_dof_list NODE_DOF",
"analysis_dof_list :",
"variable_expression : expression",
"variable_expression : discrete_pair_list",
"variable_expression : discrete_pair_list '+'",
"discrete_pair_list : discrete_pair_list ',' discrete_pair",
"discrete_pair_list : discrete_pair_list discrete_pair",
"discrete_pair_list : discrete_pair",
"discrete_pair : '(' constant_expression ',' constant_expression ')'",
"constant_expression : expression",
"expression : expression '?' if_action expression ':' else_action expression",
"expression : expression OR or_action expression",
"expression : expression AND and_action expression",
"expression : expression '|' expression",
"expression : expression '^' expression",
"expression : expression '&' expression",
"expression : expression EQUALS expression",
"expression : expression NEQUAL expression",
"expression : expression '<' expression",
"expression : expression '>' expression",
"expression : expression LT_EQ expression",
"expression : expression GT_EQ expression",
"expression : expression LSHIFT expression",
"expression : expression RSHIFT expression",
"expression : expression '+' expression",
"expression : expression '-' expression",
"expression : expression '*' expression",
"expression : expression '/' expression",
"expression : expression '%' expression",
"expression : '+' expression",
"expression : '-' expression",
"expression : '!' expression",
"expression : '~' expression",
"expression : '(' expression ')'",
"expression : INTEGER",
"expression : DOUBLE",
"expression : TIME",
"expression : function",
"function : SIN '(' expression ')'",
"function : COS '(' expression ')'",
"function : TAN '(' expression ')'",
"function : POW '(' expression ',' expression ')'",
"function : EXP '(' expression ')'",
"function : LOG '(' expression ')'",
"function : LOG10 '(' expression ')'",
"function : SQRT '(' expression ')'",
"function : HYPOT '(' expression ',' expression ')'",
"function : FLOOR '(' expression ')'",
"function : CEIL '(' expression ')'",
"function : FMOD '(' expression ',' expression ')'",
"function : FABS '(' expression ')'",
"if_action :",
"else_action :",
"or_action :",
"and_action :",
};
#endif
#define yyclearin (yychar=(-1))
#define yyerrok (yyerrflag=0)
#ifdef YYSTACKSIZE
#ifndef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#endif
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH 500
#endif
#endif
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short yyss[YYSTACKSIZE];
YYSTYPE yyvs[YYSTACKSIZE];
#define yystacksize YYSTACKSIZE
#define YYABORT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse()
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register char *yys;
    extern char *getenv();

    if (yys = getenv("YYDEBUG"))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if (yyn = yydefred[yystate]) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("yydebug: state %d, reading %d (%s)\n", yystate,
                    yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("yydebug: state %d, shifting to state %d\n",
                    yystate, yytable[yyn]);
#endif
        if (yyssp >= yyss + yystacksize - 1)
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#ifdef lint
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#ifdef lint
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("yydebug: state %d, error recovery shifting\
 to state %d\n", *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yyss + yystacksize - 1)
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("yydebug: error recovery discarding state %d\n",
                            *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("yydebug: state %d, error recovery discards token %d (%s)\n",
                    yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("yydebug: state %d, reducing by rule %d (%s)\n",
                yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 3:
#line 149 "parser.y"
{
		last_x = 0;
		last_y = 0;
		last_z = 0;
		last_material = NULL;
		last_constraint = NULL;
	    }
break;
case 8:
#line 175 "parser.y"
{
		free (problem.title);
		problem.title = yyvsp[0].s;
	    }
break;
case 9:
#line 181 "parser.y"
{
		problem.num_nodes = yyvsp[0].i;
	    }
break;
case 10:
#line 186 "parser.y"
{
		problem.num_elements = yyvsp[0].i;
	    }
break;
case 11:
#line 191 "parser.y"
{
		problem.mode = yyvsp[0].i;
	    }
break;
case 27:
#line 239 "parser.y"
{
		if (yyvsp[0].i < 1 || yyvsp[0].i > problem.num_nodes) {
		    error ("node number %u is illegal", yyvsp[0].i);
		    node = &dummy_node;
		    break;
		}

		node = CreateNode (yyvsp[0].i);
		found = TreeInsert (problem.node_tree, node);

		if (found != (Item) node) {
		    error ("node number %u is repeated", yyvsp[0].i);
		    DestroyNode (node);
		    node = &dummy_node;
		    break;
		}

		node -> x = last_x;
		node -> y = last_y;
		node -> z = last_z;
		node -> constraint = (Constraint) last_constraint;
	    }
break;
case 30:
#line 272 "parser.y"
{
		node -> x = last_x = yyvsp[0].d;
	    }
break;
case 31:
#line 277 "parser.y"
{
		node -> y = last_y = yyvsp[0].d;
	    }
break;
case 32:
#line 282 "parser.y"
{
		node -> z = last_z = yyvsp[0].d;
	    }
break;
case 33:
#line 287 "parser.y"
{
		node -> force = (Force) yyvsp[0].s;
	    }
break;
case 34:
#line 292 "parser.y"
{
		node -> constraint = (Constraint) (last_constraint = yyvsp[0].s);
	    }
break;
case 37:
#line 309 "parser.y"
{
		definition = defnlookup (yyvsp[0].s);
		Deallocate (yyvsp[0].s);
		if (!definition)
		    return 1;
	    }
break;
case 41:
#line 331 "parser.y"
{
		if (yyvsp[0].i < 1 || yyvsp[0].i > problem.num_elements) {
		    error ("element number %u is illegal", yyvsp[0].i);
		    element = &dummy_element;
		    break;
		}

		element = CreateElement (yyvsp[0].i, definition);
		found = TreeInsert (problem.element_tree, element);

		if (found != (Item) element) {
		    error ("element number %u is repeated", yyvsp[0].i);
		    DestroyElement (element);
		    element = &dummy_element;
		    break;
		}

		element -> material = (Material) last_material;
	    }
break;
case 44:
#line 361 "parser.y"
{
		unsigned i;
		unsigned size;
		unsigned number;


		if (element == &dummy_element)
		    break;

		size = int_ptr - int_array;

		if (size != element -> definition -> numnodes) {
		    number = element -> number;
		    error ("incorrect number of nodes for element %u", number);
		    break;
		}

		for (i = 1; i <= size; i ++)
		    element -> node [i] = (Node) int_array [i - 1];
	    }
break;
case 45:
#line 383 "parser.y"
{
		element -> material = (Material) (last_material = yyvsp[0].s);
	    }
break;
case 48:
#line 395 "parser.y"
{
		*int_ptr ++ = yyvsp[0].i;
	    }
break;
case 49:
#line 400 "parser.y"
{
		*int_ptr ++ = yyvsp[0].i;
	    }
break;
case 50:
#line 405 "parser.y"
{
		int_ptr = int_array;
		*int_ptr ++ = yyvsp[0].i;
	    }
break;
case 51:
#line 414 "parser.y"
{
		if (yyvsp[0].i > problem.num_nodes)
		    error ("node number %u is illegal", yyvsp[0].i);
	    }
break;
case 52:
#line 423 "parser.y"
{
		if (element -> numdistributed == 3) {
		    error ("element %u has too many loads", element -> number);
		    break;
		}

		element -> distributed [++ element -> numdistributed] =
		  (Distributed) yyvsp[0].s;
	    }
break;
case 53:
#line 434 "parser.y"
{
		if (element -> numdistributed == 3) {
		    error ("element %u has too many loads", element -> number);
		    break;
		}

		element -> distributed [++ element -> numdistributed] =
		  (Distributed) yyvsp[0].s;
	    }
break;
case 54:
#line 445 "parser.y"
{
		element -> numdistributed = 0;
		element -> distributed [++ element -> numdistributed] =
		  (Distributed) yyvsp[0].s;
	    }
break;
case 59:
#line 473 "parser.y"
{
		material = CreateMaterial (yyvsp[0].s);
		found = TreeInsert (problem.material_tree, material);

		if (found != (Item) material) {
		    error ("material %s is previously defined", yyvsp[0].s);
		    DestroyMaterial (material);
		    material = &dummy_material;
		}
	    }
break;
case 62:
#line 494 "parser.y"
{
		material -> E = yyvsp[0].d;
	    }
break;
case 63:
#line 499 "parser.y"
{
		material -> Ix = yyvsp[0].d;
	    }
break;
case 64:
#line 504 "parser.y"
{
		material -> Iy = yyvsp[0].d;
	    }
break;
case 65:
#line 509 "parser.y"
{
		material -> Iz = yyvsp[0].d;
	    }
break;
case 66:
#line 514 "parser.y"
{
		material -> A = yyvsp[0].d;
	    }
break;
case 67:
#line 519 "parser.y"
{
		material -> J = yyvsp[0].d;
	    }
break;
case 68:
#line 524 "parser.y"
{
		material -> G = yyvsp[0].d;
	    }
break;
case 69:
#line 529 "parser.y"
{
		material -> t = yyvsp[0].d;
	    }
break;
case 70:
#line 534 "parser.y"
{
		material -> rho = yyvsp[0].d;
	    }
break;
case 71:
#line 539 "parser.y"
{
		material -> nu = yyvsp[0].d;
	    }
break;
case 72:
#line 544 "parser.y"
{
		material -> kappa = yyvsp[0].d;
	    }
break;
case 73:
#line 549 "parser.y"
{
		material -> Rk = yyvsp[0].d;
	    }
break;
case 74:
#line 554 "parser.y"
{
		material -> Rm = yyvsp[0].d;
	    }
break;
case 80:
#line 582 "parser.y"
{
		load = CreateDistributed (yyvsp[0].s, 0);
		found = TreeInsert (problem.distributed_tree, load);

		if (found != (Item) load) {
		    error ("load %s is previously defined", yyvsp[0].s);
		    DestroyDistributed (load);
		    load = &dummy_load;
		}
	    }
break;
case 83:
#line 603 "parser.y"
{
		load -> direction = yyvsp[0].i;
	    }
break;
case 84:
#line 608 "parser.y"
{
		unsigned i;
		unsigned size;


		if (load == &dummy_load)
		    break;

		size = pair_ptr - pair_array;

		if (!(load -> value = Allocate (Pair, size)))
		    Fatal ("unable to allocate memory for pairs");

		UnitOffset (load -> value);
		load -> nvalues = size;

		for (i = 1; i <= size; i ++)
		    load -> value [i] = pair_array [i - 1];
	    }
break;
case 86:
#line 634 "parser.y"
{
		*pair_ptr ++ = yyvsp[0].p;
	    }
break;
case 87:
#line 639 "parser.y"
{
		*pair_ptr ++ = yyvsp[0].p;
	    }
break;
case 88:
#line 644 "parser.y"
{
		pair_ptr = pair_array;
		*pair_ptr ++ = yyvsp[0].p;
	    }
break;
case 89:
#line 653 "parser.y"
{
		if (yyvsp[-3].i < 1 || yyvsp[-3].i > problem.num_nodes)
		    error ("node number %u is illegal", yyvsp[-3].i);

		yyval.p.node = yyvsp[-3].i;
		yyval.p.magnitude = yyvsp[-1].d;
	    }
break;
case 90:
#line 662 "parser.y"
{
		if (yyvsp[-2].i < 1 || yyvsp[-2].i > problem.num_nodes)
		    error ("node number %u is illegal", yyvsp[-2].i);

		yyval.p.node = yyvsp[-2].i;
		yyval.p.magnitude = yyvsp[-1].d;
	    }
break;
case 95:
#line 692 "parser.y"
{
		force = CreateForce (yyvsp[0].s);
		found = TreeInsert (problem.force_tree, force);

		if (found != (Item) force) {
		    error ("force %s is previously defined", yyvsp[0].s);
		    DestroyForce (force);
		    force = &dummy_force;
		}
	    }
break;
case 98:
#line 713 "parser.y"
{
		AssignForce (force, Fx, InCore, copy_input (0));
	    }
break;
case 99:
#line 718 "parser.y"
{
		AssignForce (force, Fy, InCore, copy_input (0));
	    }
break;
case 100:
#line 723 "parser.y"
{
		AssignForce (force, Fz, InCore, copy_input (0));
	    }
break;
case 101:
#line 728 "parser.y"
{
		AssignForce (force, Mx, InCore, copy_input (0));
	    }
break;
case 102:
#line 733 "parser.y"
{
		AssignForce (force, My, InCore, copy_input (0));
	    }
break;
case 103:
#line 738 "parser.y"
{
		AssignForce (force, Mz, InCore, copy_input (0));
	    }
break;
case 109:
#line 766 "parser.y"
{
		constraint = CreateConstraint (yyvsp[0].s);
		found = TreeInsert (problem.constraint_tree, constraint);

		if (found != (Item) constraint) {
		    error ("constraint %s is previously defined", yyvsp[0].s);
		    DestroyConstraint (constraint);
		    constraint = &dummy_constraint;
		}
	    }
break;
case 112:
#line 787 "parser.y"
{
		constraint -> dx [Tx] = yyvsp[0].p.magnitude;
		constraint -> constraint [Tx] = yyvsp[0].p.node;
	    }
break;
case 113:
#line 793 "parser.y"
{
		constraint -> dx [Ty] = yyvsp[0].p.magnitude;
		constraint -> constraint [Ty] = yyvsp[0].p.node;
	    }
break;
case 114:
#line 799 "parser.y"
{
		constraint -> dx [Tz] = yyvsp[0].p.magnitude;
		constraint -> constraint [Tz] = yyvsp[0].p.node;
	    }
break;
case 115:
#line 805 "parser.y"
{
		constraint -> dx [Rx] = yyvsp[0].p.magnitude;
		constraint -> constraint [Rx] = yyvsp[0].p.node;
	    }
break;
case 116:
#line 811 "parser.y"
{
		constraint -> dx [Ry] = yyvsp[0].p.magnitude;
		constraint -> constraint [Ry] = yyvsp[0].p.node;
	    }
break;
case 117:
#line 817 "parser.y"
{
		constraint -> dx [Rz] = yyvsp[0].p.magnitude;
		constraint -> constraint [Rz] = yyvsp[0].p.node;
	    }
break;
case 118:
#line 823 "parser.y"
{
		constraint -> vx [Tx] = yyvsp[0].d;
	    }
break;
case 119:
#line 828 "parser.y"
{
		constraint -> vx [Ty] = yyvsp[0].d;
	    }
break;
case 120:
#line 833 "parser.y"
{
		constraint -> vx [Tz] = yyvsp[0].d;
	    }
break;
case 121:
#line 838 "parser.y"
{
		constraint -> ax [Tx] = yyvsp[0].d;
	    }
break;
case 122:
#line 843 "parser.y"
{
		constraint -> ax [Ty] = yyvsp[0].d;
	    }
break;
case 123:
#line 848 "parser.y"
{
		constraint -> ax [Tz] = yyvsp[0].d;
	    }
break;
case 125:
#line 858 "parser.y"
{
		yyval.p.magnitude = yyvsp[0].d;
		yyval.p.node = 1;
	    }
break;
case 126:
#line 864 "parser.y"
{
		yyval.p.magnitude = 0;
		yyval.p.node = yyvsp[0].i;
	    }
break;
case 127:
#line 873 "parser.y"
{
		yyval.p.magnitude = yyvsp[0].d;
		yyval.p.node = 1;
	    }
break;
case 128:
#line 879 "parser.y"
{
		yyval.p.magnitude = 0;
		yyval.p.node = yyvsp[0].i;
	    }
break;
case 129:
#line 885 "parser.y"
{
		yyval.p.magnitude = 0;
		yyval.p.node = yyvsp[0].i;
	    }
break;
case 133:
#line 907 "parser.y"
{
		analysis.alpha = yyvsp[0].d;
	    }
break;
case 134:
#line 912 "parser.y"
{
		analysis.beta = yyvsp[0].d;
	    }
break;
case 135:
#line 917 "parser.y"
{
		analysis.gamma = yyvsp[0].d;
	    }
break;
case 136:
#line 922 "parser.y"
{
		analysis.dt = yyvsp[0].d;
	    }
break;
case 137:
#line 927 "parser.y"
{
		analysis.duration = yyvsp[0].d;
	    }
break;
case 138:
#line 932 "parser.y"
{
		unsigned i;


		analysis.numnodes = int_ptr - int_array;

		if (!(analysis.nodes = Allocate (unsigned, analysis.numnodes)))
		    Fatal ("unable to allocate memory for analysis nodes");

		UnitOffset (analysis.nodes);
		for (i = 1; i <= analysis.numnodes; i ++)
		    analysis.nodes [i] = int_array [i - 1];
	    }
break;
case 139:
#line 947 "parser.y"
{
		int i;


		analysis.numdofs = 0;
		for (i = 1; i <= 6; i ++)
		    if (analysis.dofs [i])
			analysis.dofs [++ analysis.numdofs] = i;
	    }
break;
case 140:
#line 958 "parser.y"
{
		analysis.mass_mode = yyvsp[0].i;
	    }
break;
case 142:
#line 968 "parser.y"
{
		*int_ptr ++ = yyvsp[0].i;
	    }
break;
case 143:
#line 973 "parser.y"
{
		*int_ptr ++ = yyvsp[0].i;
	    }
break;
case 144:
#line 978 "parser.y"
{
		int_ptr = int_array;
	    }
break;
case 145:
#line 986 "parser.y"
{
		analysis.dofs [yyvsp[0].i] = 1;
	    }
break;
case 146:
#line 991 "parser.y"
{
		analysis.dofs [yyvsp[0].i] = 1;
	    }
break;
case 147:
#line 996 "parser.y"
{
		int i;


		for (i = 1; i <= 6; i ++)
		     analysis.dofs [i] = 0;
	    }
break;
case 148:
#line 1011 "parser.y"
{
		EmitCode (HaltOp);
		SetIP (0);
	    }
break;
case 149:
#line 1017 "parser.y"
{
		if (table_error)
		    EmitCode (PushOp, 0.0);
		else
		    EmitCode (TableOp, table, table_count);

		EmitCode (HaltOp);
		table_count = 0;
		table_error = 0;
		last_time = 0;
		SetIP (0);
	    }
break;
case 150:
#line 1031 "parser.y"
{
		if (table_error)
		    EmitCode (PushOp, 0.0);
		else
		    EmitCode (CycleOp, table, table_count);

		EmitCode (HaltOp);
		table_count = 0;
		table_error = 0;
		last_time = 0;
		SetIP (0);
	    }
break;
case 154:
#line 1055 "parser.y"
{
		if (yyvsp[-3].d < last_time) {
		    error ("point not in nondecreasing order");
		    table_error = 1;
		    break;
		}

		if (table_count == table_size) {
		    table_size = table_size ? table_size << 1 : 8;
		    if (!Reallocate (table, double, table_size))
			Fatal ("unable to expand table");
		}

		table [table_count ++] = last_time = yyvsp[-3].d;
		table [table_count ++] = yyvsp[-1].d;
	    }
break;
case 155:
#line 1077 "parser.y"
{
		EmitCode (HaltOp);
		SetIP (0);
		yyval.d = EvalCode (InCore, 0.0);
	    }
break;
case 156:
#line 1087 "parser.y"
{
		int ip = GetIP ( );
		SetIP (ip - yyvsp[0].i - 2);
		EmitCode (JmpOp, yyvsp[0].i);
		SetIP (GetIP ( ) - yyvsp[-3].i - 4);
		EmitCode (JzOp, yyvsp[-3].i + 2);
		SetIP (ip);
		yyval.i = yyvsp[-6].i + yyvsp[-4].i + yyvsp[-3].i + yyvsp[-1].i + yyvsp[0].i;
	    }
break;
case 157:
#line 1098 "parser.y"
{
		int ip = GetIP ( );
		SetIP (ip - yyvsp[0].i - 3);
		EmitCode (JnzOp, yyvsp[0].i + 1);
		SetIP (ip);
		EmitCode (TestOp);
		yyval.i = yyvsp[-3].i + yyvsp[-1].i + yyvsp[0].i + 1;
	    }
break;
case 158:
#line 1108 "parser.y"
{
		int ip = GetIP ( );
		SetIP (ip - yyvsp[0].i - 3);
		EmitCode (JzOp, yyvsp[0].i + 1);
		SetIP (ip);
		EmitCode (TestOp);
		yyval.i = yyvsp[-3].i + yyvsp[-1].i + yyvsp[0].i + 1;
	    }
break;
case 159:
#line 1118 "parser.y"
{
		EmitCode (OrOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 160:
#line 1124 "parser.y"
{
		EmitCode (XorOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 161:
#line 1130 "parser.y"
{
		EmitCode (AndOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 162:
#line 1136 "parser.y"
{
		EmitCode (EqOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 163:
#line 1142 "parser.y"
{
		EmitCode (NeqOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 164:
#line 1148 "parser.y"
{
		EmitCode (LtOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 165:
#line 1154 "parser.y"
{
		EmitCode (GtOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 166:
#line 1160 "parser.y"
{
		EmitCode (LteqOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 167:
#line 1166 "parser.y"
{
		EmitCode (GteqOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 168:
#line 1172 "parser.y"
{
		EmitCode (LsftOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 169:
#line 1178 "parser.y"
{
		EmitCode (RsftOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 170:
#line 1184 "parser.y"
{
		EmitCode (AddOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 171:
#line 1190 "parser.y"
{
		EmitCode (SubOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 172:
#line 1196 "parser.y"
{
		EmitCode (MulOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 173:
#line 1202 "parser.y"
{
		EmitCode (DivOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 174:
#line 1208 "parser.y"
{
		EmitCode (ModOp);
		yyval.i = yyvsp[-2].i + 1 + yyvsp[0].i;
	    }
break;
case 175:
#line 1214 "parser.y"
{
		yyval.i = yyvsp[0].i;
	    }
break;
case 176:
#line 1219 "parser.y"
{
		EmitCode (NegOp);
		yyval.i = 1 + yyvsp[0].i;
	    }
break;
case 177:
#line 1225 "parser.y"
{
		EmitCode (NotOp);
		yyval.i = 1 + yyvsp[0].i;
	    }
break;
case 178:
#line 1231 "parser.y"
{
		EmitCode (InvOp);
		yyval.i = 1 + yyvsp[0].i;
	    }
break;
case 179:
#line 1237 "parser.y"
{
		yyval.i = yyvsp[-1].i;
	    }
break;
case 180:
#line 1242 "parser.y"
{
		EmitCode (PushOp, (double) yyvsp[0].i);
		yyval.i = 2;
	    }
break;
case 181:
#line 1248 "parser.y"
{
		EmitCode (PushOp, yyvsp[0].d);
		yyval.i = 2;
	    }
break;
case 182:
#line 1254 "parser.y"
{
		EmitCode (TimeOp);
		yyval.i = 1;
	    }
break;
case 184:
#line 1265 "parser.y"
{
		EmitCode (SinOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 185:
#line 1271 "parser.y"
{
		EmitCode (CosOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 186:
#line 1277 "parser.y"
{
		EmitCode (TanOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 187:
#line 1283 "parser.y"
{
		EmitCode (PowOp);
		yyval.i = yyvsp[-3].i + yyvsp[-1].i + 1;
	    }
break;
case 188:
#line 1289 "parser.y"
{
		EmitCode (ExpOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 189:
#line 1295 "parser.y"
{
		EmitCode (LnOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 190:
#line 1301 "parser.y"
{
		EmitCode (LogOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 191:
#line 1307 "parser.y"
{
		EmitCode (SqrtOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 192:
#line 1313 "parser.y"
{
		EmitCode (HypotOp);
		yyval.i = yyvsp[-3].i + yyvsp[-1].i + 1;
	    }
break;
case 193:
#line 1319 "parser.y"
{
		EmitCode (FloorOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 194:
#line 1325 "parser.y"
{
		EmitCode (CeilOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 195:
#line 1331 "parser.y"
{
		EmitCode (FmodOp);
		yyval.i = yyvsp[-3].i + yyvsp[-1].i + 1;
	    }
break;
case 196:
#line 1337 "parser.y"
{
		EmitCode (FabsOp);
		yyval.i = yyvsp[-1].i + 1;
	    }
break;
case 197:
#line 1346 "parser.y"
{
		EmitCode (JzOp, 0);
		yyval.i = 2;
	    }
break;
case 198:
#line 1355 "parser.y"
{
		EmitCode (JmpOp, 0);
		yyval.i = 2;
	    }
break;
case 199:
#line 1364 "parser.y"
{
		EmitCode (CopyOp);
		EmitCode (JnzOp, 0);
		EmitCode (PopOp);
		yyval.i = 4;
	    }
break;
case 200:
#line 1375 "parser.y"
{
		EmitCode (CopyOp);
		EmitCode (JzOp, 0);
		EmitCode (PopOp);
		yyval.i = 4;
	    }
break;
#line 2563 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("yydebug: after reduction, shifting from state 0 to\
 state %d\n", YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("yydebug: state %d, reading %d (%s)\n",
                        YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("yydebug: after reduction, shifting from state %d \
to state %d\n", *yyssp, yystate);
#endif
    if (yyssp >= yyss + yystacksize - 1)
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
